@@ -32,7 +32,6 @@ RED="$ESC[0;31m"
 PYTHON2=$(which python || true)
 PYTHON3=$(which python3 || true)
 PYTHON=${PYTHON2:-$PYTHON3}
-BASE_PIP_URL=${BASE_PIP_URL:-"https://pypi.python.org/simple"}
 VIRTUALENV_VERSION="15.1.0"
 VIRTUALENV_URL="https://raw.github.com/pypa/virtualenv/$VIRTUALENV_VERSION/virtualenv.py"
 
@@ -179,16 +178,16 @@ $GREEN  -R, --no-recreate     $NO_COLOR Do not remove target directory if it alr
 $GREEN  -y, --yes             $NO_COLOR Do not ask for confirmation: assume a 'yes' reply
                          to every question.
 $GREEN  -D, --dbtype TYPE     $NO_COLOR Select the database type. TYPE can be one of
-                         'sqlite', 'mysql', 'postgres'.
+                         'sqlite', 'mysql', 'postgresql'.
                          Default: sqlite
 $GREEN  --db-user USER        $NO_COLOR Database user to use. Only used when --dbtype
-                         is either 'mysql' or 'postgres'.
+                         is either 'mysql' or 'postgresql'.
 $GREEN  --db-password PASSWORD$NO_COLOR Password of the database user. Only used when
-                         --dbtype is either 'mysql' or 'postgres'.
+                         --dbtype is either 'mysql' or 'postgresql'.
 $GREEN  --db-host HOST        $NO_COLOR Database host. Only used when --dbtype is
-                         either 'mysql' or 'postgres'
+                         either 'mysql' or 'postgresql'
 $GREEN  --db-name NAME        $NO_COLOR Name of the database. Only used when --dbtype is
-                         either 'mysql' or 'postgres'
+                         either 'mysql' or 'postgresql'
 $GREEN  -p, --python EXE      $NO_COLOR The python interpreter to use. Default: $PYTHON
 $GREEN  --develop             $NO_COLOR Install Rally with editable source code try.
                          (Default: false)
@@ -282,7 +281,7 @@ get_pkg_manager () {
     if have_command apt-get; then
         # Debian/Ubuntu
         if [ "$ASKCONFIRMATION" -eq 0 ]; then
-            pkg_manager="apt-get install --yes"
+            pkg_manager="apt-get install --yes --force-yes"
         else
             pkg_manager="apt-get install"
         fi
@@ -398,7 +397,7 @@ install_db_connector () {
         mysql)
             pip install pymysql
             ;;
-        postgres)
+        postgresql)
             pip install psycopg2
             ;;
     esac
@@ -456,7 +455,7 @@ setup_rally_configuration () {
     sed "s|#connection *=.*|connection = \"$DBCONNSTRING\"|" "$ETCDIR"/rally.conf > "$CONF_TMPFILE"
     cat "$CONF_TMPFILE" > "$ETCDIR"/rally.conf
     rm "$CONF_TMPFILE"
-    rally-manage db recreate
+    rally db recreate
 }
 
 rally_venv () {
@@ -536,7 +535,7 @@ do
             shift
             DBTYPE=$1
             case $DBTYPE in
-                sqlite|mysql|postgres);;
+                sqlite|mysql|postgresql);;
                 *)
                     print_usage | die $EX_USAGE \
                         "An invalid option has been detected."
@@ -638,6 +637,8 @@ __EOF__
     DBAUTH="$DBUSER:$DBPASSWORD@$DBHOST"
     if [ "$DBTYPE" = 'mysql' ]; then
         DBCONNSTRING="$DBTYPE+pymysql://$DBAUTH/$DBNAME"
+    elif  [ "$DBTYPE" = 'postgresql' ]; then
+        DBCONNSTRING="$DBTYPE+psycopg2://$DBAUTH/$DBNAME"
     else
         DBCONNSTRING="$DBTYPE://$DBAUTH/$DBNAME"
     fi
@@ -732,11 +733,11 @@ install_db_connector
 # Install rally
 cd "$SOURCEDIR"
 # Get latest available pip and reset shell cache
-pip install -i "$BASE_PIP_URL" -U 'pip!=8'
+pip install -U 'pip!=8'
 hash -r
 
 # Install dependencies
-pip install -i "$BASE_PIP_URL" pbr 'tox<=1.6.1' bindep
+pip install pbr 'tox<=1.6.1' bindep
 
 # Install binary dependencies
 install_required_sw "$(which_missing_packages)"
@@ -745,9 +746,9 @@ install_required_sw "$(which_missing_packages)"
 pip uninstall -y rally || true
 # Install rally
 if [ "$DEVELOPMENT_MODE" = "true" ]; then
-    pip install -i "$BASE_PIP_URL" -e .
+    pip install -e .
 else
-    pip install -i "$BASE_PIP_URL" .
+    pip install .
 fi
 
 cd "$ORIG_WD"
