@@ -16,8 +16,8 @@
 SQLAlchemy implementation for DB.API
 """
 
+import collections
 import datetime as dt
-import json
 import os
 import time
 
@@ -29,15 +29,15 @@ from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import session as db_session
 from oslo_utils import timeutils
+import six
 from sqlalchemy import or_
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.orm import load_only as sa_loadonly
 
-from rally.common.db import api as db_api
 from rally.common.db.sqlalchemy import models
-from rally.common.i18n import _
 from rally import consts
 from rally import exceptions
+from rally.task.processing import charts
 
 
 CONF = cfg.CONF
@@ -47,6 +47,45 @@ _FACADE = None
 INITIAL_REVISION_UUID = "ca3626f62937"
 
 
+def serialize_data(data):
+    if data is None:
+        return None
+    if isinstance(data, (six.integer_types,
+                         six.string_types,
+                         six.text_type,
+                         dt.date,
+                         dt.time,
+                         float,
+                         )):
+        return data
+    if isinstance(data, dict):
+        return collections.OrderedDict((k, serialize_data(v))
+                                       for k, v in data.items())
+    if isinstance(data, (list, tuple)):
+        return [serialize_data(i) for i in data]
+    if hasattr(data, "_as_dict"):
+        # NOTE(andreykurilin): it is an instance of the Model. It support a
+        #   method `_as_dict`, which should transform an object into dict
+        #   (quite logical as from the method name), BUT it does some extra
+        #   work - tries to load properties which were marked to not be loaded
+        #   in particular request and fails since the session object is not
+        #   present. That is why the code bellow makes a custom transformation.
+        result = {}
+        for key in data.__dict__:
+            if not key.startswith("_"):
+                result[key] = serialize_data(getattr(data, key))
+        return result
+
+    raise ValueError("Can not serialize %s" % data)
+
+
+def serialize(fn):
+    def wrapper(*args, **kwargs):
+        result = fn(*args, **kwargs)
+        return serialize_data(result)
+    return wrapper
+
+
 def _create_facade_lazily():
     global _FACADE
 
@@ -139,7 +178,7 @@ class Connection(object):
         #                schema, it will only add the new tables, but leave
         #                existing as is. So we should avoid of this situation.
         if self.schema_revision(engine=engine) is not None:
-            raise db_exc.DbMigrationError("DB schema is already under version"
+            raise db_exc.DBMigrationError("DB schema is already under version"
                                           " control. Use upgrade() instead")
 
         models.BASE.metadata.create_all(engine)
@@ -176,12 +215,12 @@ class Connection(object):
             return isinstance(obj, type) and issubclass(obj, models.RallyBase)
 
         if not issubclassof_rally_base(model):
-            raise Exception(_("The model should be a subclass of RallyBase"))
+            raise Exception("The model should be a subclass of RallyBase")
 
         return query
 
-    def _tags_get(self, uuid, tag_type):
-        tags = (self.model_query(models.Tag).
+    def _tags_get(self, uuid, tag_type, session=None):
+        tags = (self.model_query(models.Tag, session=session).
                 filter_by(uuid=uuid, type=tag_type).all())
 
         return list(set(t.tag for t in tags))
@@ -201,83 +240,38 @@ class Connection(object):
         task = pre_query.filter_by(uuid=uuid).first()
         if not task:
             raise exceptions.TaskNotFound(uuid=uuid)
+        task.tags = sorted(self._tags_get(uuid, consts.TagType.TASK, session))
         return task
 
-    def _make_old_task(self, task):
-        tags = self._tags_get(task.uuid, consts.TagType.TASK)
-        tag = tags[0] if tags else ""
-
-        return {
-            "id": task.id,
-            "uuid": task.uuid,
-            "deployment_uuid": task.deployment_uuid,
-            "status": task.status,
-            "created_at": task.created_at,
-            "updated_at": task.updated_at,
-            "tag": tag,
-            "verification_log": json.dumps(task.validation_result)
-        }
-
-    def _make_old_task_result(self, workload, workload_data_list):
-        raw_data = [data
-                    for workload_data in workload_data_list
-                    for data in workload_data.chunk_data["raw"]]
-        return {
-            "id": workload.id,
-            "task_uuid": workload.task_uuid,
-            "created_at": workload.created_at,
-            "updated_at": workload.updated_at,
-            "key": {
-                "name": workload.name,
-                "pos": workload.position,
-                "kw": {
-                    "args": workload.args,
-                    "runner": workload.runner,
-                    "context": workload.context,
-                    "sla": workload.sla,
-                    "hooks": [r["config"] for r in workload.hooks],
-                }
-            },
-            "data": {
-                "raw": raw_data,
-                "load_duration": workload.load_duration,
-                "full_duration": workload.full_duration,
-                "sla": workload.sla_results["sla"],
-                "hooks": workload.hooks
-            }
-        }
-
     def _task_workload_data_get_all(self, workload_uuid):
-        return (self.model_query(models.WorkloadData).
-                filter_by(workload_uuid=workload_uuid).
-                order_by(models.WorkloadData.chunk_order.asc()))
+        session = get_session()
+        with session.begin():
+            results = (self.model_query(models.WorkloadData, session=session).
+                       filter_by(workload_uuid=workload_uuid).
+                       order_by(models.WorkloadData.chunk_order.asc()))
 
-    # @db_api.serialize
-    def task_get(self, uuid):
-        task = self._task_get(uuid)
-        return self._make_old_task(task)
+        return sorted([raw for workload_data in results
+                       for raw in workload_data.chunk_data["raw"]],
+                      key=lambda x: x["timestamp"])
+
+    @serialize
+    def task_get(self, uuid=None, detailed=False):
+        session = get_session()
+        task = serialize_data(self._task_get(uuid, session=session))
+
+        if detailed:
+            task["subtasks"] = self._subtasks_get_all_by_task_uuid(
+                uuid, session=session)
 
-    # @db_api.serialize
-    def task_get_detailed(self, uuid):
-        task = self.task_get(uuid)
-        task["results"] = self._task_result_get_all_by_uuid(uuid)
         return task
 
-    @db_api.serialize
+    @serialize
     def task_get_status(self, uuid):
         return self._task_get(uuid, load_only="status").status
 
-    # @db_api.serialize
-    def task_get_detailed_last(self):
-        task = (self.model_query(models.Task).
-                order_by(models.Task.id.desc()).first())
-        task = self._make_old_task(task)
-        task["results"] = self._task_result_get_all_by_uuid(task["uuid"])
-        return task
-
-    # @db_api.serialize
+    @serialize
     def task_create(self, values):
-        new_tag = values.pop("tag", None)
+        tags = values.pop("tags", None)
         # TODO(ikhudoshyn): currently 'input_task'
         # does not come in 'values'
         # After completely switching to the new
@@ -290,36 +284,35 @@ class Connection(object):
         task.update(values)
         task.save()
 
-        if new_tag:
-            tag = models.Tag()
-            tag.update({
-                "uuid": task.uuid,
-                "type": consts.TagType.TASK,
-                "tag": new_tag
-            })
-            tag.save()
+        if tags:
+            for t in set(tags):
+                tag = models.Tag()
+                tag.update({"uuid": task.uuid,
+                            "type": consts.TagType.TASK,
+                            "tag": t})
+                tag.save()
+        task.tags = sorted(self._tags_get(task.uuid, consts.TagType.TASK))
+        return task
 
-        return self._make_old_task(task)
-
-    # @db_api.serialize
+    @serialize
     def task_update(self, uuid, values):
         session = get_session()
         values.pop("uuid", None)
-        new_tag = values.pop("tag", None)
+        tags = values.pop("tags", None)
         with session.begin():
             task = self._task_get(uuid, session=session)
             task.update(values)
 
-            if new_tag:
-                tag = models.Tag()
-                tag.update({
-                    "uuid": uuid,
-                    "type": consts.TagType.TASK,
-                    "tag": new_tag
-                })
-                tag.save()
-
-        return self._make_old_task(task)
+            if tags:
+                for t in set(tags):
+                    tag = models.Tag()
+                    tag.update({"uuid": task.uuid,
+                                "type": consts.TagType.TASK,
+                                "tag": t})
+                    tag.save()
+            # take an updated instance of task
+            task = self._task_get(uuid, session=session)
+        return task
 
     def task_update_status(self, uuid, statuses, status_value):
         session = get_session()
@@ -331,27 +324,41 @@ class Connection(object):
         )
         if not result:
             status = " or ".join(statuses)
-            msg = _("Task with uuid='%(uuid)s' and in statuses:'"
-                    "%(statuses)s' not found.'") % {"uuid": uuid,
-                                                    "statuses": status}
+            msg = ("Task with uuid='%(uuid)s' and in statuses:'"
+                   "%(statuses)s' not found.'") % {"uuid": uuid,
+                                                   "statuses": status}
             raise exceptions.RallyException(msg)
         return result
 
-    # @db_api.serialize
-    def task_list(self, status=None, deployment=None):
-        query = self.model_query(models.Task)
+    @serialize
+    def task_list(self, status=None, deployment=None, tags=None):
+        session = get_session()
+        tasks = []
+        with session.begin():
+            query = self.model_query(models.Task)
 
-        filters = {}
-        if status is not None:
-            filters["status"] = status
-        if deployment is not None:
-            filters["deployment_uuid"] = self.deployment_get(
-                deployment)["uuid"]
+            filters = {}
+            if status is not None:
+                filters["status"] = status
+            if deployment is not None:
+                filters["deployment_uuid"] = self.deployment_get(
+                    deployment)["uuid"]
+            if filters:
+                query = query.filter_by(**filters)
 
-        if filters:
-            query = query.filter_by(**filters)
+            if tags:
+                uuids = self._uuids_by_tags_get(
+                    consts.TagType.TASK, tags)
+                if not uuids:
+                    return []
+                query = query.filter(models.Task.uuid.in_(uuids))
 
-        return [self._make_old_task(task) for task in query.all()]
+            for task in query.all():
+                task.tags = sorted(
+                    self._tags_get(task.uuid, consts.TagType.TASK, session))
+                tasks.append(task)
+
+        return tasks
 
     def task_delete(self, uuid, status=None):
         session = get_session()
@@ -384,26 +391,23 @@ class Connection(object):
                                                            actual=task.status)
                 raise exceptions.TaskNotFound(uuid=uuid)
 
-    def _task_result_get_all_by_uuid(self, uuid):
-        results = []
+    def _subtasks_get_all_by_task_uuid(self, task_uuid, session=None):
+        result = (self.model_query(models.Subtask, session=session).filter_by(
+            task_uuid=task_uuid).all())
+        subtasks = []
+        for subtask in result:
+            subtask = serialize_data(subtask)
+            subtask["workloads"] = []
+            workloads = (self.model_query(models.Workload, session=session).
+                         filter_by(subtask_uuid=subtask["uuid"]).all())
+            for workload in workloads:
+                workload.data = self._task_workload_data_get_all(
+                    workload.uuid)
+                subtask["workloads"].append(serialize_data(workload))
+            subtasks.append(subtask)
+        return subtasks
 
-        workloads = (self.model_query(models.Workload).
-                     filter_by(task_uuid=uuid).all())
-
-        for workload in workloads:
-            workload_data_list = self._task_workload_data_get_all(
-                workload.uuid)
-
-            results.append(
-                self._make_old_task_result(workload, workload_data_list))
-
-        return results
-
-    # @db_api.serialize
-    def task_result_get_all_by_uuid(self, uuid):
-        return self._task_result_get_all_by_uuid(uuid)
-
-    @db_api.serialize
+    @serialize
     def subtask_create(self, task_uuid, title, description=None, context=None):
         subtask = models.Subtask(task_uuid=task_uuid)
         subtask.update({
@@ -414,25 +418,40 @@ class Connection(object):
         subtask.save()
         return subtask
 
-    @db_api.serialize
-    def workload_create(self, task_uuid, subtask_uuid, key):
+    @serialize
+    def subtask_update(self, subtask_uuid, values):
+        subtask = self.model_query(models.Subtask).filter_by(
+            uuid=subtask_uuid).first()
+        subtask.update(values)
+        subtask.save()
+        return subtask
+
+    @serialize
+    def workload_get(self, workload_uuid):
+        return self.model_query(models.Workload).filter_by(
+            uuid=workload_uuid).first()
+
+    @serialize
+    def workload_create(self, task_uuid, subtask_uuid, name, description,
+                        position, runner, runner_type, hooks, context, sla,
+                        args, context_execution, statistics):
         workload = models.Workload(task_uuid=task_uuid,
-                                   subtask_uuid=subtask_uuid)
-        workload.update({
-            "name": key["name"],
-            "position": key["pos"],
-            "runner": key["kw"]["runner"],
-            "runner_type": key["kw"]["runner"]["type"],
-            "context": key["kw"].get("context", {}),
-            "sla": key["kw"].get("sla", {}),
-            "args": key["kw"].get("args", {}),
-            "context_execution": {},
-            "statistics": {},
-        })
+                                   subtask_uuid=subtask_uuid,
+                                   name=name,
+                                   description=description,
+                                   position=position,
+                                   runner=runner,
+                                   runner_type=runner_type,
+                                   hooks=hooks,
+                                   context=context,
+                                   sla=sla,
+                                   args=args,
+                                   context_execution=context_execution,
+                                   statistics=statistics)
         workload.save()
         return workload
 
-    @db_api.serialize
+    @serialize
     def workload_data_create(self, task_uuid, workload_uuid, chunk_order,
                              data):
         workload_data = models.WorkloadData(task_uuid=task_uuid,
@@ -481,74 +500,74 @@ class Connection(object):
         workload_data.save()
         return workload_data
 
-    @db_api.serialize
-    def workload_set_results(self, workload_uuid, data):
-        workload = self.model_query(models.Workload).filter_by(
-            uuid=workload_uuid).first()
+    @serialize
+    def workload_set_results(self, workload_uuid, subtask_uuid, task_uuid,
+                             load_duration, full_duration, start_time,
+                             sla_results, hooks_results):
+        session = get_session()
+        with session.begin():
+            workload_results = self._task_workload_data_get_all(workload_uuid)
 
-        workload_data_list = self._task_workload_data_get_all(workload.uuid)
+            iter_count = len(workload_results)
 
-        raw_data = [raw
-                    for workload_data in workload_data_list
-                    for raw in workload_data.chunk_data["raw"]]
-        iter_count = len(raw_data)
+            failed_iter_count = 0
+            max_duration = None
+            min_duration = None
 
-        failed_iter_count = 0
-        max_duration = 0
-        min_duration = 0
+            for d in workload_results:
+                if d.get("error"):
+                    failed_iter_count += 1
 
-        success = True
+                duration = d.get("duration", 0)
 
-        for d in raw_data:
-            if d.get("error"):
-                failed_iter_count += 1
+                if max_duration is None or duration > max_duration:
+                    max_duration = duration
 
-            duration = d.get("duration", 0)
+                if min_duration is None or min_duration > duration:
+                    min_duration = duration
 
-            if duration > max_duration:
-                max_duration = duration
+            durations_stat = charts.MainStatsTable(
+                {"total_iteration_count": iter_count})
 
-            if min_duration and min_duration > duration:
-                min_duration = duration
+            for itr in workload_results:
+                durations_stat.add_iteration(itr)
 
-        sla = data.get("sla", [])
-        # TODO(ikhudoshyn): if no SLA was specified and there are
-        # failed iterations is it success?
-        # NOTE(ikhudoshyn): we call it 'pass_sla'
-        # for the sake of consistency with other models
-        # so if no SLAs were specified, then we assume pass_sla == True
-        success = all([s.get("success") for s in sla])
+            sla = sla_results or []
+            # NOTE(ikhudoshyn): we call it 'pass_sla'
+            # for the sake of consistency with other models
+            # so if no SLAs were specified, then we assume pass_sla == True
+            success = all([s.get("success") for s in sla])
 
-        now = timeutils.utcnow()
-        delta = dt.timedelta(seconds=data.get("full_duration", 0))
-        start = now - delta
+            session.query(models.Workload).filter_by(
+                uuid=workload_uuid).update(
+                {
+                    "sla_results": {"sla": sla},
+                    "context_execution": {},
+                    "hooks": hooks_results or [],
+                    "load_duration": load_duration,
+                    "full_duration": full_duration,
+                    "min_duration": min_duration,
+                    "max_duration": max_duration,
+                    "total_iteration_count": iter_count,
+                    "failed_iteration_count": failed_iter_count,
+                    "start_time": start_time,
+                    "statistics": {"durations": durations_stat.to_dict()},
+                    "pass_sla": success}
+            )
+            task_values = {
+                "task_duration": models.Task.task_duration + load_duration}
+            if not success:
+                task_values["pass_sla"] = False
 
-        workload.update({
-            "task_uuid": workload.task_uuid,
-            "subtask_uuid": workload.subtask_uuid,
-            "sla_results": {"sla": sla},
-            "context_execution": {},
-            "hooks": data.get("hooks", []),
-            "load_duration": data.get("load_duration", 0),
-            "full_duration": data.get("full_duration", 0),
-            "min_duration": min_duration,
-            "max_duration": max_duration,
-            "total_iteration_count": iter_count,
-            "failed_iteration_count": failed_iter_count,
-            # TODO(ikhudoshyn)
-            "start_time": start,
-            "statistics": {},
-            "pass_sla": success
-        })
+            subtask_values = {
+                "duration": models.Subtask.duration + load_duration}
+            if not success:
+                subtask_values["pass_sla"] = False
+            session.query(models.Task).filter_by(uuid=task_uuid).update(
+                task_values)
 
-        # TODO(ikhudoshyn): if pass_sla is False,
-        # then update task's and subtask's pass_sla
-
-        # TODO(ikhudoshyn): update task.task_duration
-        # and subtask.duration
-
-        workload.save()
-        return workload
+            session.query(models.Subtask).filter_by(uuid=subtask_uuid).update(
+                subtask_values)
 
     def _deployment_get(self, deployment, session=None):
         stored_deployment = self.model_query(
@@ -563,7 +582,7 @@ class Connection(object):
             raise exceptions.DeploymentNotFound(deployment=deployment)
         return stored_deployment
 
-    @db_api.serialize
+    @serialize
     def deployment_create(self, values):
         deployment = models.Deployment()
         try:
@@ -586,11 +605,11 @@ class Connection(object):
             if not count:
                 raise exceptions.DeploymentNotFound(deployment=uuid)
 
-    @db_api.serialize
+    @serialize
     def deployment_get(self, deployment):
         return self._deployment_get(deployment)
 
-    @db_api.serialize
+    @serialize
     def deployment_update(self, deployment, values):
         session = get_session()
         values.pop("uuid", None)
@@ -599,7 +618,7 @@ class Connection(object):
             dpl.update(values)
         return dpl
 
-    @db_api.serialize
+    @serialize
     def deployment_list(self, status=None, parent_uuid=None, name=None):
         query = (self.model_query(models.Deployment).
                  filter_by(parent_uuid=parent_uuid))
@@ -610,14 +629,14 @@ class Connection(object):
             query = query.filter_by(status=status)
         return query.all()
 
-    @db_api.serialize
+    @serialize
     def resource_create(self, values):
         resource = models.Resource()
         resource.update(values)
         resource.save()
         return resource
 
-    @db_api.serialize
+    @serialize
     def resource_get_all(self, deployment_uuid, provider_name=None, type=None):
         query = (self.model_query(models.Resource).
                  filter_by(deployment_uuid=deployment_uuid))
@@ -633,18 +652,18 @@ class Connection(object):
         if not count:
             raise exceptions.ResourceNotFound(id=id)
 
-    @db_api.serialize
-    def verifier_create(self, name, vtype, namespace, source, version,
+    @serialize
+    def verifier_create(self, name, vtype, platform, source, version,
                         system_wide, extra_settings=None):
         verifier = models.Verifier()
-        properties = {"name": name, "type": vtype, "namespace": namespace,
+        properties = {"name": name, "type": vtype, "platform": platform,
                       "source": source, "extra_settings": extra_settings,
                       "version": version, "system_wide": system_wide}
         verifier.update(properties)
         verifier.save()
         return verifier
 
-    @db_api.serialize
+    @serialize
     def verifier_get(self, verifier_id):
         return self._verifier_get(verifier_id)
 
@@ -657,7 +676,7 @@ class Connection(object):
             raise exceptions.ResourceNotFound(id=verifier_id)
         return verifier
 
-    @db_api.serialize
+    @serialize
     def verifier_list(self, status=None):
         query = self.model_query(models.Verifier)
         if status:
@@ -675,7 +694,7 @@ class Connection(object):
             if not count:
                 raise exceptions.ResourceNotFound(id=verifier_id)
 
-    @db_api.serialize
+    @serialize
     def verifier_update(self, verifier_id, properties):
         session = get_session()
         with session.begin():
@@ -684,7 +703,7 @@ class Connection(object):
             verifier.save()
         return verifier
 
-    @db_api.serialize
+    @serialize
     def verification_create(self, verifier_id, deployment_id, tags=None,
                             run_args=None):
         verifier = self._verifier_get(verifier_id)
@@ -705,7 +724,7 @@ class Connection(object):
 
         return verification
 
-    @db_api.serialize
+    @serialize
     def verification_get(self, verification_uuid):
         verification = self._verification_get(verification_uuid)
         verification.tags = sorted(self._tags_get(verification.uuid,
@@ -720,7 +739,7 @@ class Connection(object):
             raise exceptions.ResourceNotFound(id=verification_uuid)
         return verification
 
-    @db_api.serialize
+    @serialize
     def verification_list(self, verifier_id=None, deployment_id=None,
                           tags=None, status=None):
         session = get_session()
@@ -762,7 +781,7 @@ class Connection(object):
         if not count:
             raise exceptions.ResourceNotFound(id=verification_uuid)
 
-    @db_api.serialize
+    @serialize
     def verification_update(self, verification_uuid, properties):
         session = get_session()
         with session.begin():
@@ -771,7 +790,7 @@ class Connection(object):
             verification.save()
         return verification
 
-    @db_api.serialize
+    @serialize
     def register_worker(self, values):
         try:
             worker = models.Worker()
@@ -783,7 +802,7 @@ class Connection(object):
             raise exceptions.WorkerAlreadyRegistered(
                 worker=values["hostname"])
 
-    @db_api.serialize
+    @serialize
     def get_worker(self, hostname):
         try:
             return (self.model_query(models.Worker).
