@@ -32,7 +32,6 @@ import uuid
 
 from six import moves
 
-from rally.common.i18n import _LE
 from rally.common import logging
 from rally import exceptions
 
@@ -47,7 +46,7 @@ class ImmutableMixin(object):
 
     def __setattr__(self, key, value):
         if self._inited:
-            raise exceptions.ImmutableException()
+            raise AttributeError("This object is immutable.")
         super(ImmutableMixin, self).__setattr__(key, value)
 
 
@@ -101,8 +100,15 @@ class Timer(object):
         if type:
             self.error = (type, value, tb)
 
-    def duration(self):
-        return self.finish - self.start
+    def duration(self, fmt=False):
+        duration = self.finish - self.start
+        if not fmt:
+            return duration
+        if duration > 60:
+            return "%.2f min" % (duration / 60)
+        if duration > 0.1:
+            return "%.2f sec" % duration
+        return "%.2f msec" % (duration * 1000)
 
 
 class Struct(object):
@@ -264,6 +270,14 @@ class RandomNameGeneratorMixin(object):
     RESOURCE_NAME_FORMAT = "rally_XXXXXXXX_XXXXXXXX"
     RESOURCE_NAME_ALLOWED_CHARACTERS = string.ascii_letters + string.digits
 
+    @classmethod
+    def _get_resource_name_format(cls):
+        return cls.RESOURCE_NAME_FORMAT
+
+    @classmethod
+    def _get_resource_name_allowed_characters(cls):
+        return cls.RESOURCE_NAME_ALLOWED_CHARACTERS
+
     @classmethod
     def _generate_random_part(cls, length):
         """Generate a random string.
@@ -273,8 +287,9 @@ class RandomNameGeneratorMixin(object):
                   containing only characters from
                   cls.RESOURCE_NAME_ALLOWED_CHARACTERS
         """
-        return "".join(random.choice(cls.RESOURCE_NAME_ALLOWED_CHARACTERS)
-                       for i in range(length))
+        return "".join(
+            random.choice(cls._get_resource_name_allowed_characters())
+            for i in range(length))
 
     @classmethod
     def _generate_task_id_part(cls, task_id, length):
@@ -284,16 +299,16 @@ class RandomNameGeneratorMixin(object):
 
         if len(task_id_part) < length:
             LOG.debug("Task ID %(task_id)s cannot be included in a random "
-                      "name because it is too short. Format: %(format)s" %
-                      {"task_id": task_id,
-                       "format": cls.RESOURCE_NAME_FORMAT})
-        elif any(char not in cls.RESOURCE_NAME_ALLOWED_CHARACTERS
+                      "name because it is too short. Format: %(format)s"
+                      % {"task_id": task_id,
+                         "format": cls._get_resource_name_format()})
+        elif any(char not in cls._get_resource_name_allowed_characters()
                  for char in task_id_part):
             LOG.debug("Task ID %(task_id)s cannot be included in a random "
                       "name because it includes disallowed characters. "
-                      "Allowed characters are: %(chars)s" %
-                      {"task_id": task_id,
-                       "chars": cls.RESOURCE_NAME_ALLOWED_CHARACTERS})
+                      "Allowed characters are: %(chars)s"
+                      % {"task_id": task_id,
+                         "chars": cls._get_resource_name_allowed_characters()})
         else:
             return task_id_part
 
@@ -309,6 +324,12 @@ class RandomNameGeneratorMixin(object):
         finally:
             random.seed()
 
+    def get_owner_id(self):
+        if hasattr(self, "task"):
+            return self.task["uuid"]
+        elif hasattr(self, "verification"):
+            return self.verification["uuid"]
+
     def generate_random_name(self):
         """Generate pseudo-random resource name for scenarios.
 
@@ -323,16 +344,13 @@ class RandomNameGeneratorMixin(object):
 
         :returns: str, pseudo-random name
         """
-        if hasattr(self, "task"):
-            task_id = self.task["uuid"]
-        elif hasattr(self, "verification"):
-            task_id = self.verification["uuid"]
+        task_id = self.get_owner_id()
 
         match = self._resource_name_placeholder_re.match(
-            self.RESOURCE_NAME_FORMAT)
+            self._get_resource_name_format())
         if match is None:
             raise ValueError("%s is not a valid resource name format" %
-                             self.RESOURCE_NAME_FORMAT)
+                             self._get_resource_name_format())
         parts = match.groupdict()
         return "".join([
             parts["prefix"],
@@ -357,13 +375,13 @@ class RandomNameGeneratorMixin(object):
         :returns: bool
         """
         match = cls._resource_name_placeholder_re.match(
-            cls.RESOURCE_NAME_FORMAT)
+            cls._get_resource_name_format())
         parts = match.groupdict()
         subst = {
             "prefix": re.escape(parts["prefix"]),
             "sep": re.escape(parts["sep"]),
             "suffix": re.escape(parts["suffix"]),
-            "chars": re.escape(cls.RESOURCE_NAME_ALLOWED_CHARACTERS),
+            "chars": re.escape(cls._get_resource_name_allowed_characters()),
             "rand_length": len(parts["rand"])}
         if task_id:
             subst["task_id"] = cls._generate_task_id_part(task_id,
@@ -399,13 +417,34 @@ def name_matches_object(name, *objects, **kwargs):
     """
     unique_rng_options = {}
     for obj in objects:
-        key = (obj.RESOURCE_NAME_FORMAT, obj.RESOURCE_NAME_ALLOWED_CHARACTERS)
+        key = (obj._get_resource_name_format(),
+               obj._get_resource_name_allowed_characters())
         if key not in unique_rng_options:
             unique_rng_options[key] = obj
     return any(obj.name_matches_object(name, **kwargs)
                for obj in unique_rng_options.values())
 
 
+def make_name_matcher(*names):
+    """Construct a matcher for custom names
+
+    In case of contexts, there can be custom names. To reuse common cleanup
+    mechanism for cleaning up such resources, this method creates a subclass of
+    RandomNameGeneratorMixin with customized `name_matches_object` method.
+    """
+    class CustomNameMatcher(RandomNameGeneratorMixin):
+        # generate unique string to guarantee processing that custom names
+        RESOURCE_NAME_FORMAT = str(uuid.uuid4())
+
+        NAMES = names
+
+        @classmethod
+        def name_matches_object(cls, name, task_id=None, exact=True):
+            return name in cls.NAMES
+
+    return CustomNameMatcher
+
+
 def merge(length, *sources):
     """Merge lists of lists.
 
@@ -745,8 +784,8 @@ class BackupHelper(object):
     def backup(self, original_path):
         if original_path in self._stored_data:
             raise exceptions.RallyException(
-                _LE("Failed to back up %s since it was already stored.") %
-                original_path)
+                "Failed to back up %s since it was already stored."
+                % original_path)
         backup_path = generate_random_path(self._tempdir)
         LOG.debug("Creating backup of %s in %s" % (original_path, backup_path))
         try:
@@ -788,3 +827,25 @@ class BackupHelper(object):
             if os.path.exists(path):
                 LOG.debug("Deleting %s" % path)
                 shutil.rmtree(path)
+
+
+def prettify_xml(elem, level=0):
+    """Adds indents.
+
+    Code of this method was copied from
+        http://effbot.org/zone/element-lib.htm#prettyprint
+
+    """
+    i = "\n" + level * "  "
+    if len(elem):
+        if not elem.text or not elem.text.strip():
+            elem.text = i + "  "
+        if not elem.tail or not elem.tail.strip():
+            elem.tail = i
+        for elem in elem:
+            prettify_xml(elem, level + 1)
+        if not elem.tail or not elem.tail.strip():
+            elem.tail = i
+    else:
+        if level and (not elem.tail or not elem.tail.strip()):
+            elem.tail = i
