@@ -12,19 +12,23 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from rally.common.i18n import _
 from rally.common import logging
+from rally.common import utils
+from rally.common import validation
 from rally import consts
-from rally import osclients
+from rally.plugins.openstack.cleanup import manager as resource_manager
+from rally.plugins.openstack import osclients
+from rally.plugins.openstack.services.storage import block
 from rally.task import context
 
 
 LOG = logging.getLogger(__name__)
 
 
-@context.configure(name="volume_types", order=410)
+@validation.add("required_platform", platform="openstack", admin=True)
+@context.configure(name="volume_types", platform="openstack", order=410)
 class VolumeTypeGenerator(context.Context):
-    """Context class for adding volumes types for benchmarks."""
+    """Adds cinder volumes types."""
 
     CONFIG_SCHEMA = {
         "type": "array",
@@ -32,25 +36,24 @@ class VolumeTypeGenerator(context.Context):
         "items": {"type": "string"}
     }
 
-    @logging.log_task_wrapper(LOG.info, _("Enter context: `volume_types`"))
     def setup(self):
         admin_clients = osclients.Clients(
             self.context.get("admin", {}).get("credential"),
             api_info=self.context["config"].get("api_versions"))
-        cinder = admin_clients.cinder()
+        cinder_service = block.BlockStorage(
+            admin_clients, name_generator=self.generate_random_name)
         self.context["volume_types"] = []
         for vtype_name in self.config:
             LOG.debug("Creating Cinder volume type %s" % vtype_name)
-            vtype = cinder.volume_types.create(vtype_name)
+            vtype = cinder_service.create_volume_type(vtype_name)
             self.context["volume_types"].append({"id": vtype.id,
                                                  "name": vtype_name})
 
-    @logging.log_task_wrapper(LOG.info, _("Exit context: `volume_types`"))
     def cleanup(self):
-        admin_clients = osclients.Clients(
-            self.context.get("admin", {}).get("credential"),
-            api_info=self.context["config"].get("api_versions"))
-        cinder = admin_clients.cinder()
-        for vtype in self.context["volume_types"]:
-            LOG.debug("Deleting Cinder volume type %s" % vtype["name"])
-            cinder.volume_types.delete(vtype["id"])
+        mather = utils.make_name_matcher(*self.config)
+        resource_manager.cleanup(
+            names=["cinder.volume_types"],
+            admin=self.context["admin"],
+            api_versions=self.context["config"].get("api_versions"),
+            superclass=mather,
+            task_id=self.get_owner_id())
