@@ -12,21 +12,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from rally.common.i18n import _
-from rally.common import logging
 from rally.common import utils as rutils
+from rally.common import validation
 from rally import consts
 from rally.plugins.openstack.cleanup import manager as resource_manager
 from rally.plugins.openstack.scenarios.magnum import utils as magnum_utils
+from rally.plugins.openstack.scenarios.nova import utils as nova_utils
 from rally.task import context
 
 
-LOG = logging.getLogger(__name__)
-
-
-@context.configure(name="clusters", order=480)
+@validation.add("required_platform", platform="openstack", users=True)
+@context.configure(name="clusters", platform="openstack", order=480)
 class ClusterGenerator(context.Context):
-    """Context class for generating temporary cluster for benchmarks."""
+    """Creates specified amount of Magnum clusters."""
 
     CONFIG_SCHEMA = {
         "type": "object",
@@ -45,14 +43,22 @@ class ClusterGenerator(context.Context):
 
     DEFAULT_CONFIG = {"node_count": 1}
 
-    @logging.log_task_wrapper(LOG.info, _("Enter context: `Cluster`"))
     def setup(self):
         for user, tenant_id in rutils.iterate_per_tenants(
                 self.context["users"]):
 
+            nova_scenario = nova_utils.NovaScenario({
+                "user": user,
+                "task": self.context["task"],
+                "config": {"api_versions": self.context["config"].get(
+                    "api_versions", [])}
+            })
+            keypair = nova_scenario._create_keypair()
+
             magnum_scenario = magnum_utils.MagnumScenario({
                 "user": user,
                 "task": self.context["task"],
+                "owner_id": self.context["owner_id"],
                 "config": {"api_versions": self.context["config"].get(
                     "api_versions", [])}
             })
@@ -64,11 +70,12 @@ class ClusterGenerator(context.Context):
                 ct_uuid = ctx.get("cluster_template")
             cluster = magnum_scenario._create_cluster(
                 cluster_template=ct_uuid,
-                node_count=self.config.get("node_count"))
+                node_count=self.config.get("node_count"), keypair=keypair)
             self.context["tenants"][tenant_id]["cluster"] = cluster.uuid
 
-    @logging.log_task_wrapper(LOG.info, _("Exit context: `Cluster`"))
     def cleanup(self):
         resource_manager.cleanup(
-            names=["magnum.clusters"],
-            users=self.context.get("users", []))
+            names=["magnum.clusters", "nova.keypairs"],
+            users=self.context.get("users", []),
+            superclass=magnum_utils.MagnumScenario,
+            task_id=self.get_owner_id())
