@@ -15,9 +15,8 @@
 
 from oslo_config import cfg
 
-from rally.common.i18n import _
-from rally.common import logging
 from rally.common import utils
+from rally.common import validation
 from rally import consts as rally_consts
 from rally.plugins.openstack.cleanup import manager as resource_manager
 from rally.plugins.openstack.context.manila import consts
@@ -25,12 +24,11 @@ from rally.plugins.openstack.scenarios.manila import utils as manila_utils
 from rally.task import context
 
 CONF = cfg.CONF
-LOG = logging.getLogger(__name__)
-
 CONTEXT_NAME = consts.SECURITY_SERVICES_CONTEXT_NAME
 
 
-@context.configure(name=CONTEXT_NAME, order=445)
+@validation.add("required_platform", platform="openstack", users=True)
+@context.configure(name=CONTEXT_NAME, platform="openstack", order=445)
 class SecurityServices(context.Context):
     """This context creates 'security services' for Manila project."""
 
@@ -65,8 +63,6 @@ class SecurityServices(context.Context):
         "security_services": [],
     }
 
-    @logging.log_task_wrapper(
-        LOG.info, _("Enter context: `%s`") % CONTEXT_NAME)
     def setup(self):
         for user, tenant_id in (utils.iterate_per_tenants(
                 self.context.get("users", []))):
@@ -76,6 +72,7 @@ class SecurityServices(context.Context):
             if self.config["security_services"]:
                 manila_scenario = manila_utils.ManilaScenario({
                     "task": self.task,
+                    "owner_id": self.context["owner_id"],
                     "user": user,
                     "config": {
                         "api_versions": self.context["config"].get(
@@ -87,9 +84,9 @@ class SecurityServices(context.Context):
                     self.context["tenants"][tenant_id][CONTEXT_NAME][
                         "security_services"].append(inst)
 
-    @logging.log_task_wrapper(LOG.info, _("Exit context: `%s`") % CONTEXT_NAME)
     def cleanup(self):
         resource_manager.cleanup(
             names=["manila.security_services"],
             users=self.context.get("users", []),
-        )
+            superclass=manila_utils.ManilaScenario,
+            task_id=self.get_owner_id())
