@@ -15,21 +15,18 @@
 
 import requests
 
-from rally.common.i18n import _
-from rally.common import logging
 from rally.common import utils as rutils
+from rally.common import validation
 from rally import consts
 from rally import exceptions
-from rally import osclients
 from rally.plugins.openstack.cleanup import manager as resource_manager
+from rally.plugins.openstack import osclients
 from rally.plugins.openstack.scenarios.sahara import utils
 from rally.task import context
 
 
-LOG = logging.getLogger(__name__)
-
-
-@context.configure(name="sahara_job_binaries", order=442)
+@validation.add("required_platform", platform="openstack", users=True)
+@context.configure(name="sahara_job_binaries", platform="openstack", order=442)
 class SaharaJobBinaries(context.Context):
     """Context class for setting up Job Binaries for an EDP job."""
 
@@ -77,8 +74,6 @@ class SaharaJobBinaries(context.Context):
     # downloads for each tenant
     lib_cache = {}
 
-    @logging.log_task_wrapper(LOG.info,
-                              _("Enter context: `Sahara Job Binaries`"))
     def setup(self):
         utils.init_sahara_context(self)
         for user, tenant_id in rutils.iterate_per_tenants(
@@ -109,7 +104,7 @@ class SaharaJobBinaries(context.Context):
     def setup_inputs(self, sahara, tenant_id, input_type, input_url):
         if input_type == "swift":
             raise exceptions.RallyException(
-                _("Swift Data Sources are not implemented yet"))
+                "Swift Data Sources are not implemented yet")
         # Todo(nkonovalov): Add swift credentials parameters and data upload
         input_ds = sahara.data_sources.create(
             name=self.generate_random_name(),
@@ -140,8 +135,6 @@ class SaharaJobBinaries(context.Context):
         self.context["tenants"][tenant_id]["sahara"][lib_type].append(
             job_binary.id)
 
-    @logging.log_task_wrapper(LOG.info,
-                              _("Exit context: `Sahara Job Binaries`"))
     def cleanup(self):
         resources = ["job_binary_internals", "job_binaries"]
 
