@@ -16,12 +16,20 @@
 import functools
 import random
 
-from rally import osclients
+from oslo_config import cfg
+from osprofiler import profiler
+from rally.common.plugin import plugin
+from rally.plugins.openstack import osclients
+from rally.task import context
 from rally.task import scenario
 
-configure = functools.partial(scenario.configure, namespace="openstack")
+configure = functools.partial(scenario.configure, platform="openstack")
+
+CONF = cfg.CONF
 
 
+@context.add_default_context("users@openstack", {})
+@plugin.default_meta(inherit=False)
 class OpenStackScenario(scenario.Scenario):
     """Base class for all OpenStack scenarios."""
 
@@ -29,8 +37,8 @@ class OpenStackScenario(scenario.Scenario):
         super(OpenStackScenario, self).__init__(context)
         if context:
             api_info = {}
-            if "api_versions" in context.get("config", {}):
-                api_versions = context["config"]["api_versions"]
+            if "api_versions@openstack" in context.get("config", {}):
+                api_versions = context["config"]["api_versions@openstack"]
                 for service in api_versions:
                     api_info[service] = {
                         "version": api_versions[service].get("version"),
@@ -54,6 +62,8 @@ class OpenStackScenario(scenario.Scenario):
         if clients:
             self._clients = clients
 
+        self._init_profiler(context)
+
     def _choose_user(self, context):
         """Choose one user from users context
 
@@ -81,8 +91,7 @@ class OpenStackScenario(scenario.Scenario):
     def clients(self, client_type, version=None):
         """Returns a python openstack client of the requested type.
 
-        The client will be that for one of the temporary non-administrator
-        users created before the benchmark launch.
+        Only one non-admin user is used per every run of scenario.
 
         :param client_type: Client type ("nova"/"glance" etc.)
         :param version: client version ("1"/"2" etc.)
@@ -105,12 +114,26 @@ class OpenStackScenario(scenario.Scenario):
 
         return client(version) if version is not None else client()
 
-    @classmethod
-    def validate(cls, name, config, admin=None, users=None, deployment=None):
-        if admin:
-            admin = osclients.Clients(admin)
-        if users:
-            users = [osclients.Clients(user["credential"]) for user in users]
-        super(OpenStackScenario, cls).validate(
-            name=name, config=config, admin=admin, users=users,
-            deployment=deployment)
+    def _init_profiler(self, context):
+        """Inits the profiler."""
+        if not CONF.openstack.enable_profiler:
+            return
+        if context is not None:
+            cred = None
+            profiler_hmac_key = None
+            if context.get("admin"):
+                cred = context["admin"]["credential"]
+                if cred.profiler_hmac_key is not None:
+                    profiler_hmac_key = cred.profiler_hmac_key
+            if context.get("user"):
+                cred = context["user"]["credential"]
+                if cred.profiler_hmac_key is not None:
+                    profiler_hmac_key = cred.profiler_hmac_key
+            if profiler_hmac_key is None:
+                return
+            profiler.init(profiler_hmac_key)
+            trace_id = profiler.get().get_base_id()
+            self.add_output(complete={
+                "title": "OSProfiler Trace-ID",
+                "chart_plugin": "TextArea",
+                "data": [trace_id]})
