@@ -17,49 +17,47 @@ import random
 
 from oslo_config import cfg
 
+from rally.common import logging
 from rally import exceptions
 from rally.plugins.openstack import scenario
+from rally.plugins.openstack.services.storage import block
 from rally.plugins.openstack.wrappers import cinder as cinder_wrapper
 from rally.plugins.openstack.wrappers import glance as glance_wrapper
 from rally.task import atomic
 from rally.task import utils as bench_utils
 
-CINDER_BENCHMARK_OPTS = [
-    cfg.FloatOpt("cinder_volume_create_prepoll_delay",
-                 default=2.0,
-                 help="Time to sleep after creating a resource before"
-                      " polling for it status"),
-    cfg.FloatOpt("cinder_volume_create_timeout",
-                 default=600.0,
-                 help="Time to wait for cinder volume to be created."),
-    cfg.FloatOpt("cinder_volume_create_poll_interval",
-                 default=2.0,
-                 help="Interval between checks when waiting for volume"
-                      " creation."),
-    cfg.FloatOpt("cinder_volume_delete_timeout",
-                 default=600.0,
-                 help="Time to wait for cinder volume to be deleted."),
-    cfg.FloatOpt("cinder_volume_delete_poll_interval",
-                 default=2.0,
-                 help="Interval between checks when waiting for volume"
-                      " deletion."),
-    cfg.FloatOpt("cinder_backup_restore_timeout",
-                 default=600.0,
-                 help="Time to wait for cinder backup to be restored."),
-    cfg.FloatOpt("cinder_backup_restore_poll_interval",
-                 default=2.0,
-                 help="Interval between checks when waiting for backup"
-                      " restoring."),
-]
-
 CONF = cfg.CONF
-benchmark_group = cfg.OptGroup(name="benchmark", title="benchmark options")
-CONF.register_opts(CINDER_BENCHMARK_OPTS, group=benchmark_group)
+LOG = logging.getLogger(__name__)
+
+
+class CinderBasic(scenario.OpenStackScenario):
+    def __init__(self, context=None, admin_clients=None, clients=None):
+        super(CinderBasic, self).__init__(context, admin_clients, clients)
+        if hasattr(self, "_admin_clients"):
+            self.admin_cinder = block.BlockStorage(
+                self._admin_clients, name_generator=self.generate_random_name,
+                atomic_inst=self.atomic_actions())
+        if hasattr(self, "_clients"):
+            self.cinder = block.BlockStorage(
+                self._clients, name_generator=self.generate_random_name,
+                atomic_inst=self.atomic_actions())
+
+    def get_random_server(self):
+        server_id = random.choice(self.context["tenant"]["servers"])
+        return self.clients("nova").servers.get(server_id)
 
 
 class CinderScenario(scenario.OpenStackScenario):
     """Base class for Cinder scenarios with basic atomic actions."""
 
+    def __init__(self, context=None, admin_clients=None, clients=None):
+        super(CinderScenario, self).__init__(context, admin_clients, clients)
+        LOG.warning(
+            "Class %s is deprecated since Rally 0.10.0 and will be removed "
+            "soon. Use "
+            "rally.plugins.openstack.services.storage.block.BlockStorage "
+            "instead." % self.__class__)
+
     @atomic.action_timer("cinder.list_volumes")
     def _list_volumes(self, detailed=True):
         """Returns user volumes list."""
@@ -141,7 +139,7 @@ class CinderScenario(scenario.OpenStackScenario):
                 to_del = keys[i * delete_size:(i + 1) * delete_size]
                 self.clients("cinder").volumes.delete_metadata(volume, to_del)
 
-    @atomic.optional_action_timer("cinder.create_volume")
+    @atomic.action_timer("cinder.create_volume")
     def _create_volume(self, size, **kwargs):
         """Create one volume.
 
@@ -152,9 +150,6 @@ class CinderScenario(scenario.OpenStackScenario):
                      dictionary, must contain two values:
                          min - minimum size volumes will be created as;
                          max - maximum size volumes will be created as.
-        :param atomic_action: True if this is an atomic action. added
-                              and handled by the
-                              optional_action_timer() decorator
         :param kwargs: Other optional parameters to initialize the volume
         :returns: Created volume object
         """
@@ -166,14 +161,14 @@ class CinderScenario(scenario.OpenStackScenario):
 
         # NOTE(msdubov): It is reasonable to wait 5 secs before starting to
         #                check whether the volume is ready => less API calls.
-        self.sleep_between(CONF.benchmark.cinder_volume_create_prepoll_delay)
+        self.sleep_between(CONF.openstack.cinder_volume_create_prepoll_delay)
 
-        volume = bench_utils.wait_for(
+        volume = bench_utils.wait_for_status(
             volume,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         return volume
 
@@ -217,8 +212,8 @@ class CinderScenario(scenario.OpenStackScenario):
             ready_statuses=["deleted"],
             check_deletion=True,
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_delete_timeout,
-            check_interval=CONF.benchmark.cinder_volume_delete_poll_interval
+            timeout=CONF.openstack.cinder_volume_delete_timeout,
+            check_interval=CONF.openstack.cinder_volume_delete_poll_interval
         )
 
     @atomic.action_timer("cinder.extend_volume")
@@ -239,12 +234,12 @@ class CinderScenario(scenario.OpenStackScenario):
             new_size = random.randint(new_size["min"], new_size["max"])
 
         volume.extend(volume, new_size)
-        volume = bench_utils.wait_for(
+        volume = bench_utils.wait_for_status(
             volume,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
 
     @atomic.action_timer("cinder.upload_volume_to_image")
@@ -267,22 +262,22 @@ class CinderScenario(scenario.OpenStackScenario):
                                            container_format, disk_format)
         # NOTE (e0ne): upload_to_image changes volume status to uploading so
         # we need to wait until it will be available.
-        volume = bench_utils.wait_for(
+        volume = bench_utils.wait_for_status(
             volume,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         image_id = img["os-volume_upload_image"]["image_id"]
         image = self.clients("glance").images.get(image_id)
         wrapper = glance_wrapper.wrap(self._clients.glance, self)
-        image = bench_utils.wait_for(
+        image = bench_utils.wait_for_status(
             image,
             ready_statuses=["active"],
             update_resource=wrapper.get_image,
-            timeout=CONF.benchmark.glance_image_create_timeout,
-            check_interval=CONF.benchmark.glance_image_create_poll_interval
+            timeout=CONF.openstack.glance_image_create_timeout,
+            check_interval=CONF.openstack.glance_image_create_poll_interval
         )
 
         return image
@@ -305,13 +300,13 @@ class CinderScenario(scenario.OpenStackScenario):
         client = cinder_wrapper.wrap(self._clients.cinder, self)
         snapshot = client.create_snapshot(volume_id, **kwargs)
 
-        self.sleep_between(CONF.benchmark.cinder_volume_create_prepoll_delay)
-        snapshot = bench_utils.wait_for(
+        self.sleep_between(CONF.openstack.cinder_volume_create_prepoll_delay)
+        snapshot = bench_utils.wait_for_status(
             snapshot,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         return snapshot
 
@@ -329,8 +324,8 @@ class CinderScenario(scenario.OpenStackScenario):
             ready_statuses=["deleted"],
             check_deletion=True,
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_delete_timeout,
-            check_interval=CONF.benchmark.cinder_volume_delete_poll_interval
+            timeout=CONF.openstack.cinder_volume_delete_timeout,
+            check_interval=CONF.openstack.cinder_volume_delete_poll_interval
         )
 
     @atomic.action_timer("cinder.create_backup")
@@ -341,12 +336,12 @@ class CinderScenario(scenario.OpenStackScenario):
         :param kwargs: Other optional parameters
         """
         backup = self.clients("cinder").backups.create(volume_id, **kwargs)
-        return bench_utils.wait_for(
+        return bench_utils.wait_for_status(
             backup,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
 
     @atomic.action_timer("cinder.delete_backup")
@@ -363,8 +358,8 @@ class CinderScenario(scenario.OpenStackScenario):
             ready_statuses=["deleted"],
             check_deletion=True,
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_delete_timeout,
-            check_interval=CONF.benchmark.cinder_volume_delete_poll_interval
+            timeout=CONF.openstack.cinder_volume_delete_timeout,
+            check_interval=CONF.openstack.cinder_volume_delete_poll_interval
         )
 
     @atomic.action_timer("cinder.restore_backup")
@@ -377,19 +372,19 @@ class CinderScenario(scenario.OpenStackScenario):
         restore = self.clients("cinder").restores.restore(backup_id, volume_id)
         restored_volume = self.clients("cinder").volumes.get(restore.volume_id)
         backup_for_restore = self.clients("cinder").backups.get(backup_id)
-        bench_utils.wait_for(
+        bench_utils.wait_for_status(
             backup_for_restore,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_backup_restore_timeout,
-            check_interval=CONF.benchmark.cinder_backup_restore_poll_interval
+            timeout=CONF.openstack.cinder_backup_restore_timeout,
+            check_interval=CONF.openstack.cinder_backup_restore_poll_interval
         )
-        return bench_utils.wait_for(
+        return bench_utils.wait_for_status(
             restored_volume,
             ready_statuses=["available"],
             update_resource=bench_utils.get_from_manager(),
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
 
     @atomic.action_timer("cinder.list_backups")
@@ -448,6 +443,15 @@ class CinderScenario(scenario.OpenStackScenario):
         """
         return volume_type.set_keys(metadata)
 
+    @atomic.action_timer("cinder.get_volume_type")
+    def _get_volume_type(self, volume_type):
+        """get details of volume_type.
+
+        :param volume_type: The ID of the :class:`VolumeType` to get
+        :rtype: :class:`VolumeType`
+        """
+        return self.admin_clients("cinder").volume_types.get(volume_type)
+
     @atomic.action_timer("cinder.transfer_create")
     def _transfer_create(self, volume_id):
         """Create a volume transfer.
@@ -499,4 +503,4 @@ class CinderScenario(scenario.OpenStackScenario):
         resp = self.admin_clients("cinder").volume_encryption_types.delete(
             volume_type)
         if (resp[0].status_code != 202):
-            raise exceptions.EncryptionTypeDeleteException()
+            raise exceptions.RallyException("EncryptionType Deletion Failed")
