@@ -21,10 +21,10 @@ from rally.task import validation
 """Scenarios for monasca Metrics API."""
 
 
-@validation.required_clients("monasca")
-@validation.required_services(consts.Service.MONASCA)
-@validation.required_openstack(users=True)
-@scenario.configure(name="MonascaMetrics.list_metrics")
+@validation.add("required_services",
+                services=[consts.Service.MONASCA])
+@validation.add("required_platform", platform="openstack", users=True)
+@scenario.configure(name="MonascaMetrics.list_metrics", platform="openstack")
 class ListMetrics(monascautils.MonascaScenario):
 
     def run(self, **kwargs):
