@@ -17,7 +17,6 @@ import random
 
 from oslo_config import cfg
 
-from rally.common.i18n import _
 from rally.common import logging
 from rally import exceptions
 from rally.plugins.openstack import scenario
@@ -25,22 +24,9 @@ from rally.plugins.openstack.wrappers import network as network_wrapper
 from rally.task import atomic
 from rally.task import utils
 
-NEUTRON_BENCHMARK_OPTS = [
-    cfg.FloatOpt(
-        "neutron_create_loadbalancer_timeout",
-        default=float(500),
-        help="Neutron create loadbalancer timeout"),
-    cfg.FloatOpt(
-        "neutron_create_loadbalancer_poll_interval",
-        default=float(2),
-        help="Neutron create loadbalancer poll interval")
-]
 
 CONF = cfg.CONF
-benchmark_group = cfg.OptGroup(name="benchmark",
-                               title="benchmark options")
-CONF.register_group(benchmark_group)
-CONF.register_opts(NEUTRON_BENCHMARK_OPTS, group=benchmark_group)
+
 
 LOG = logging.getLogger(__name__)
 
@@ -65,12 +51,22 @@ class NeutronScenario(scenario.OpenStackScenario):
         param kwargs: dict, network options
         returns: str, Neutron network-id
         """
-        networks = self._list_networks(atomic_action=False)
+        networks = self._list_networks()
         for net in networks:
             if (net["name"] == network) or (net["id"] == network):
                 return net["id"]
-        msg = (_("Network %s not found.") % network)
-        raise exceptions.NotFoundException(message=msg)
+        raise exceptions.NotFoundException(
+            message="Network %s not found." % network)
+
+    @property
+    def _ext_gw_mode_enabled(self):
+        """Determine if the ext-gw-mode extension is enabled.
+
+        Without this extension, we can't pass the enable_snat parameter.
+        """
+        return any(
+            e["alias"] == "ext-gw-mode"
+            for e in self.clients("neutron").list_extensions()["extensions"])
 
     @atomic.action_timer("neutron.create_network")
     def _create_network(self, network_create_args):
@@ -83,13 +79,10 @@ class NeutronScenario(scenario.OpenStackScenario):
         return self.clients("neutron").create_network(
             {"network": network_create_args})
 
-    @atomic.optional_action_timer("neutron.list_networks")
+    @atomic.action_timer("neutron.list_networks")
     def _list_networks(self, **kwargs):
         """Return user networks list.
 
-        :param atomic_action: True if this is an atomic action. added
-                              and handled by the
-                              optional_action_timer() decorator
         :param kwargs: network list options
         """
         return self.clients("neutron").list_networks(**kwargs)["networks"]
@@ -164,6 +157,17 @@ class NeutronScenario(scenario.OpenStackScenario):
         """Returns user subnetworks list."""
         return self.clients("neutron").list_subnets()["subnets"]
 
+    @atomic.action_timer("neutron.show_subnet")
+    def _show_subnet(self, subnet, **kwargs):
+        """show subnet details.
+
+        :param: subnet: Subnet object
+        :param: kwargs: Optional additional arguments for subnet show
+        :returns: details of the subnet
+        """
+        return self.clients("neutron").show_subnet(subnet["subnet"]["id"],
+                                                   **kwargs)
+
     @atomic.action_timer("neutron.update_subnet")
     def _update_subnet(self, subnet, subnet_update_args):
         """Update the neutron subnet.
@@ -200,8 +204,9 @@ class NeutronScenario(scenario.OpenStackScenario):
             for network in self._list_networks():
                 if network.get("router:external"):
                     external_network = network
-                    gw_info = {"network_id": external_network["id"],
-                               "enable_snat": True}
+                    gw_info = {"network_id": external_network["id"]}
+                    if self._ext_gw_mode_enabled:
+                        gw_info["enable_snat"] = True
                     router_create_args.setdefault("external_gateway_info",
                                                   gw_info)
 
@@ -213,6 +218,17 @@ class NeutronScenario(scenario.OpenStackScenario):
         """Returns user routers list."""
         return self.clients("neutron").list_routers()["routers"]
 
+    @atomic.action_timer("neutron.show_router")
+    def _show_router(self, router, **kwargs):
+        """Show information of a given router.
+
+        :param router: ID or name of router to look up
+        :kwargs: dict, POST /v2.0/routers show options
+        :return: details of the router
+        """
+        return self.clients("neutron").show_router(
+            router["router"]["id"], **kwargs)
+
     @atomic.action_timer("neutron.delete_router")
     def _delete_router(self, router):
         """Delete neutron router
@@ -253,6 +269,16 @@ class NeutronScenario(scenario.OpenStackScenario):
         """Return user ports list."""
         return self.clients("neutron").list_ports()["ports"]
 
+    @atomic.action_timer("neutron.show_port")
+    def _show_port(self, port, **params):
+        """Return user port details.
+
+        :param port: dict, neutron port
+        :param params: neutron port show options
+        :returns: neutron port dict
+        """
+        return self.clients("neutron").show_port(port["port"]["id"], **params)
+
     @atomic.action_timer("neutron.update_port")
     def _update_port(self, port, port_update_args):
         """Update the neutron port.
@@ -275,9 +301,9 @@ class NeutronScenario(scenario.OpenStackScenario):
         """
         self.clients("neutron").delete_port(port["port"]["id"])
 
-    @logging.log_deprecated_args(_("network_create_args is deprecated; "
-                                   "use the network context instead"),
-                                 "0.1.0", "network_create_args")
+    @logging.log_deprecated_args(
+        "network_create_args is deprecated; use the network context instead",
+        "0.1.0", "network_create_args")
     def _get_or_create_network(self, network_create_args=None):
         """Get a network from context, or create a new one.
 
@@ -296,8 +322,8 @@ class NeutronScenario(scenario.OpenStackScenario):
             return {"network":
                     random.choice(self.context["tenant"]["networks"])}
         else:
-            LOG.warning(_("Running this scenario without either the 'network' "
-                          "or 'existing_network' context is deprecated"))
+            LOG.warning("Running this scenario without either the 'network' "
+                        "or 'existing_network' context is deprecated")
             return self._create_network(network_create_args or {})
 
     def _create_subnets(self, network,
@@ -382,15 +408,35 @@ class NeutronScenario(scenario.OpenStackScenario):
         self.clients("neutron").remove_interface_router(
             router["id"], {"subnet_id": subnet["id"]})
 
-    @atomic.optional_action_timer("neutron.create_pool")
+    @atomic.action_timer("neutron.add_gateway_router")
+    def _add_gateway_router(self, router, ext_net, enable_snat):
+        """Set the external network gateway for a router.
+
+        :param router: dict, neutron router
+        :param ext_net: external network for the gateway
+        :param enable_snat: True if enable snat
+        """
+        gw_info = {"network_id": ext_net["network"]["id"]}
+        if self._ext_gw_mode_enabled:
+            gw_info["enable_snat"] = enable_snat
+        self.clients("neutron").add_gateway_router(
+            router["router"]["id"], gw_info)
+
+    @atomic.action_timer("neutron.remove_gateway_router")
+    def _remove_gateway_router(self, router):
+        """Removes an external network gateway from the specified router.
+
+        :param router: dict, neutron router
+        """
+        self.clients("neutron").remove_gateway_router(
+            router["router"]["id"])
+
+    @atomic.action_timer("neutron.create_pool")
     def _create_lb_pool(self, subnet_id, **pool_create_args):
         """Create LB pool(v1)
 
         :param subnet_id: str, neutron subnet-id
         :param pool_create_args: dict, POST /lb/pools request options
-        :param atomic_action: True if this is an atomic action. added
-                              and handled by the
-                              optional_action_timer() decorator
         :returns: dict, neutron lb pool
         """
         args = {"lb_method": self.LB_METHOD,
@@ -411,11 +457,9 @@ class NeutronScenario(scenario.OpenStackScenario):
         pools = []
         for net in networks:
             subnets.extend(net.get("subnets", []))
-        with atomic.ActionTimer(self, "neutron.create_%s_pools" %
-                                len(subnets)):
-            for subnet_id in subnets:
-                pools.append(self._create_lb_pool(
-                    subnet_id, atomic_action=False, **pool_create_args))
+        for subnet_id in subnets:
+            pools.append(self._create_lb_pool(
+                subnet_id, **pool_create_args))
         return pools
 
     @atomic.action_timer("neutron.list_pools")
@@ -514,16 +558,13 @@ class NeutronScenario(scenario.OpenStackScenario):
         """
         return self.clients("neutron").delete_floatingip(floating_ip["id"])
 
-    @atomic.optional_action_timer("neutron.create_healthmonitor")
+    @atomic.action_timer("neutron.create_healthmonitor")
     def _create_v1_healthmonitor(self, **healthmonitor_create_args):
         """Create LB healthmonitor.
 
         This atomic function creates healthmonitor with the provided
         healthmonitor_create_args.
 
-        :param atomic_action: True if this is an atomic action. added
-                              and handled by the
-                              optional_action_timer() decorator
         :param healthmonitor_create_args: dict, POST /lb/healthmonitors
         :returns: neutron healthmonitor dict
         """
@@ -594,6 +635,17 @@ class NeutronScenario(scenario.OpenStackScenario):
         """Return list of Neutron security groups."""
         return self.clients("neutron").list_security_groups(**kwargs)
 
+    @atomic.action_timer("neutron.show_security_group")
+    def _show_security_group(self, security_group, **kwargs):
+        """Show security group details.
+
+        :param: security_group: dict, neutron security_group
+        :param: kwargs: Optional additional arguments for security_group show
+        :returns: security_group details
+        """
+        return self.clients("neutron").show_security_group(
+            security_group["security_group"]["id"], **kwargs)
+
     @atomic.action_timer("neutron.update_security_group")
     def _update_security_group(self, security_group,
                                **security_group_update_args):
@@ -618,15 +670,12 @@ class NeutronScenario(scenario.OpenStackScenario):
             raise exceptions.GetResourceFailure(resource=lb, err=e)
         return new_lb["loadbalancer"]
 
-    @atomic.optional_action_timer("neutron.create_lbaasv2_loadbalancer")
+    @atomic.action_timer("neutron.create_lbaasv2_loadbalancer")
     def _create_lbaasv2_loadbalancer(self, subnet_id, **lb_create_args):
         """Create LB loadbalancer(v2)
 
         :param subnet_id: str, neutron subnet-id
         :param lb_create_args: dict, POST /lbaas/loadbalancers request options
-        :param atomic_action: True if this is an atomic action. added
-                              and handled by the
-                              optional_action_timer() decorator
         :returns: dict, neutron lb
         """
         args = {"name": self.generate_random_name(),
@@ -640,9 +689,9 @@ class NeutronScenario(scenario.OpenStackScenario):
             ready_statuses=["ACTIVE"],
             status_attr="provisioning_status",
             update_resource=self.update_loadbalancer_resource,
-            timeout=CONF.benchmark.neutron_create_loadbalancer_timeout,
+            timeout=CONF.openstack.neutron_create_loadbalancer_timeout,
             check_interval=(
-                CONF.benchmark.neutron_create_loadbalancer_poll_interval)
+                CONF.openstack.neutron_create_loadbalancer_poll_interval)
         )
         return lb
 
@@ -655,3 +704,162 @@ class NeutronScenario(scenario.OpenStackScenario):
         """
         return self.clients("neutron").list_loadbalancers(retrieve_all,
                                                           **lb_list_args)
+
+    @atomic.action_timer("neutron.create_bgpvpn")
+    def _create_bgpvpn(self, **kwargs):
+        """Create Bgpvpn resource (POST /bgpvpn/bgpvpn)
+
+        :param kwargs: optional parameters to create BGP VPN
+        :returns dict, bgpvpn resource details
+        """
+        kwargs["name"] = self.generate_random_name()
+        return self.admin_clients("neutron").create_bgpvpn({"bgpvpn": kwargs})
+
+    @atomic.action_timer("neutron.delete_bgpvpn")
+    def _delete_bgpvpn(self, bgpvpn):
+        """Delete Bgpvpn resource.(DELETE /bgpvpn/bgpvpns/{id})
+
+        :param bgpvpn: dict, bgpvpn
+        :return dict, bgpvpn
+        """
+        return self.admin_clients("neutron").delete_bgpvpn(
+            bgpvpn["bgpvpn"]["id"])
+
+    @atomic.action_timer("neutron.list_bgpvpns")
+    def _list_bgpvpns(self, **kwargs):
+        """Return bgpvpns list.
+
+        :param kwargs: dict, GET /bgpvpn/bgpvpns request options
+        :returns: bgpvpns list
+        """
+        return self.admin_clients("neutron").list_bgpvpns(
+            True, **kwargs)["bgpvpns"]
+
+    @atomic.action_timer("neutron.update_bgpvpn")
+    def _update_bgpvpn(self, bgpvpn, update_name=False, **kwargs):
+        """Update a bgpvpn.
+
+        :param bgpvpn: dict, bgpvpn
+        :param update_name: update_name: bool, whether or not to modify
+        BGP VPN name
+        :param **kwargs: dict, PUT /bgpvpn/bgpvpns update options
+        :return dict, updated bgpvpn
+        """
+        if update_name or "name" in kwargs:
+            kwargs["name"] = self.generate_random_name()
+        return self.admin_clients("neutron").update_bgpvpn(
+            bgpvpn["bgpvpn"]["id"], {"bgpvpn": kwargs})
+
+    @atomic.action_timer("neutron.create_bgpvpn_network_assoc")
+    def _create_bgpvpn_network_assoc(self, bgpvpn, network):
+        """Creates a new BGP VPN network association.
+
+        :param bgpvpn: dict, bgpvpn
+        :param network: dict, network
+        :return dict: network_association
+        """
+        netassoc = {"network_id": network["id"]}
+        return self.clients("neutron").create_bgpvpn_network_assoc(
+            bgpvpn["bgpvpn"]["id"], {"network_association": netassoc})
+
+    @atomic.action_timer("neutron.delete_bgpvpn_network_assoc")
+    def _delete_bgpvpn_network_assoc(self, bgpvpn, net_assoc):
+        """Delete the specified BGP VPN network association
+
+        :param bgpvpn: dict, bgpvpn
+        :param net_assoc: dict, network
+        :return dict: network_association
+        """
+        return self.clients("neutron").delete_bgpvpn_network_assoc(
+            bgpvpn["bgpvpn"]["id"], net_assoc["network_association"]["id"])
+
+    @atomic.action_timer("neutron.create_bgpvpn_router_assoc")
+    def _create_bgpvpn_router_assoc(self, bgpvpn, router):
+        """Creates a new BGP VPN router association.
+
+        :param bgpvpn: dict, bgpvpn
+        :param router: dict, router
+        :return dict: network_association
+        """
+        router_assoc = {"router_id": router["id"]}
+        return self.clients("neutron").create_bgpvpn_router_assoc(
+            bgpvpn["bgpvpn"]["id"], {"router_association": router_assoc})
+
+    @atomic.action_timer("neutron.delete_bgpvpn_router_assoc")
+    def _delete_bgpvpn_router_assoc(self, bgpvpn, router_assoc):
+        """Delete the specified BGP VPN router association
+
+        :param bgpvpn: dict, bgpvpn
+        :param router_assoc: dict, router
+        :return dict: router_association
+        """
+        return self.clients("neutron").delete_bgpvpn_router_assoc(
+            bgpvpn["bgpvpn"]["id"], router_assoc["router_association"]["id"])
+
+    @atomic.action_timer("neutron.list_bgpvpn_network_assocs")
+    def _list_bgpvpn_network_assocs(self, bgpvpn, **kwargs):
+        """List network association of bgpvpn
+
+        :param bgpvpn: dict, bgpvpn
+        :param **kwargs: dict, optional parameters
+        :return dict: network_association
+        """
+        return self.clients("neutron").list_bgpvpn_network_assocs(
+            bgpvpn["bgpvpn"]["id"], **kwargs)
+
+    @atomic.action_timer("neutron.list_bgpvpn_router_assocs")
+    def _list_bgpvpn_router_assocs(self, bgpvpn, **kwargs):
+        """List router association of bgpvpn
+
+        :param bgpvpn: dict, bgpvpn
+        :param **kwargs: dict, optional parameters
+        :return dict: router_association
+        """
+        return self.clients("neutron").list_bgpvpn_router_assocs(
+            bgpvpn["bgpvpn"]["id"], **kwargs)
+
+    @atomic.action_timer("neutron.create_security_group_rule")
+    def _create_security_group_rule(self, security_group_id,
+                                    **security_group_rule_args):
+        """Create Neutron security-group-rule.
+
+        param: security_group_id: id of neutron security_group
+        param: security_group_rule_args: dict, POST
+               /v2.0/security-group-rules request options
+        return: dict, neutron security-group-rule
+        """
+        security_group_rule_args["security_group_id"] = security_group_id
+        if "direction" not in security_group_rule_args:
+            security_group_rule_args["direction"] = "ingress"
+
+        return self.clients("neutron").create_security_group_rule(
+            {"security_group_rule": security_group_rule_args})
+
+    @atomic.action_timer("neutron.list_security_group_rules")
+    def _list_security_group_rules(self, **kwargs):
+        """List all security group rules.
+
+        :param kwargs: Optional additional arguments for roles list
+        :return: list of security group rules
+        """
+        return self.clients("neutron").list_security_group_rules(**kwargs)
+
+    @atomic.action_timer("neutron.show_security_group_rule")
+    def _show_security_group_rule(self, security_group_rule, **kwargs):
+        """Show information of a given security group rule.
+
+        :param security_group_rule: id of security group rule
+        :param kwargs: Optional additional arguments for roles list
+        :return: details of security group rule
+        """
+        return self.clients("neutron").show_security_group_rule(
+            security_group_rule, **kwargs)
+
+    @atomic.action_timer("neutron.delete_security_group_rule")
+    def _delete_security_group_rule(self, security_group_rule):
+        """Delete a given security group rule.
+
+        :param security_group_rule: id of security group rule
+        """
+        self.clients("neutron").delete_security_group_rule(
+            security_group_rule)
