@@ -30,14 +30,16 @@ LOG = logging.getLogger(__name__)
                worker_flavor={"type": "nova_flavor"},
                neutron_net={"type": "neutron_network"},
                floating_ip_pool={"type": "neutron_network"})
-@validation.flavor_exists("master_flavor")
-@validation.flavor_exists("worker_flavor")
-@validation.required_contexts("users", "sahara_image")
-@validation.number("workers_count", minval=1, integer_only=True)
-@validation.required_services(consts.Service.SAHARA)
-@validation.required_openstack(users=True)
-@scenario.configure(context={"cleanup": ["sahara"]},
-                    name="SaharaClusters.create_and_delete_cluster")
+@validation.add("flavor_exists", param_name="master_flavor")
+@validation.add("flavor_exists", param_name="worker_flavor")
+@validation.add("required_contexts", contexts=["users", "sahara_image"])
+@validation.add("number", param_name="workers_count", minval=1,
+                integer_only=True)
+@validation.add("required_services", services=[consts.Service.SAHARA])
+@validation.add("required_platform", platform="openstack", users=True)
+@scenario.configure(context={"cleanup@openstack": ["sahara"]},
+                    name="SaharaClusters.create_and_delete_cluster",
+                    platform="openstack")
 class CreateAndDeleteCluster(utils.SaharaScenario):
 
     def run(self, workers_count, plugin_name, hadoop_version,
@@ -120,13 +122,15 @@ class CreateAndDeleteCluster(utils.SaharaScenario):
 @types.convert(flavor={"type": "nova_flavor"},
                master_flavor={"type": "nova_flavor"},
                worker_flavor={"type": "nova_flavor"})
-@validation.flavor_exists("master_flavor")
-@validation.flavor_exists("worker_flavor")
-@validation.required_services(consts.Service.SAHARA)
-@validation.required_contexts("users", "sahara_image")
-@validation.number("workers_count", minval=1, integer_only=True)
-@scenario.configure(context={"cleanup": ["sahara"]},
-                    name="SaharaClusters.create_scale_delete_cluster")
+@validation.add("flavor_exists", param_name="master_flavor")
+@validation.add("flavor_exists", param_name="worker_flavor")
+@validation.add("required_services", services=[consts.Service.SAHARA])
+@validation.add("required_contexts", contexts=["users", "sahara_image"])
+@validation.add("number", param_name="workers_count", minval=1,
+                integer_only=True)
+@scenario.configure(context={"cleanup@openstack": ["sahara"]},
+                    name="SaharaClusters.create_scale_delete_cluster",
+                    platform="openstack")
 class CreateScaleDeleteCluster(utils.SaharaScenario):
 
     def run(self, master_flavor, worker_flavor, workers_count,
