@@ -19,7 +19,6 @@ from oslo_config import cfg
 from oslo_utils import uuidutils
 from saharaclient.api import base as sahara_base
 
-from rally.common.i18n import _
 from rally.common import logging
 from rally.common import utils as rutils
 from rally import consts
@@ -29,32 +28,10 @@ from rally.plugins.openstack.scenarios.sahara import consts as sahara_consts
 from rally.task import atomic
 from rally.task import utils
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
-SAHARA_BENCHMARK_OPTS = [
-    cfg.IntOpt("sahara_cluster_create_timeout", default=1800,
-               deprecated_name="cluster_create_timeout",
-               help="A timeout in seconds for a cluster create operation"),
-    cfg.IntOpt("sahara_cluster_delete_timeout", default=900,
-               deprecated_name="cluster_delete_timeout",
-               help="A timeout in seconds for a cluster delete operation"),
-    cfg.IntOpt("sahara_cluster_check_interval", default=5,
-               deprecated_name="cluster_check_interval",
-               help="Cluster status polling interval in seconds"),
-    cfg.IntOpt("sahara_job_execution_timeout", default=600,
-               deprecated_name="job_execution_timeout",
-               help="A timeout in seconds for a Job Execution to complete"),
-    cfg.IntOpt("sahara_job_check_interval", default=5,
-               deprecated_name="job_check_interval",
-               help="Job Execution status polling interval in seconds"),
-    cfg.IntOpt("sahara_workers_per_proxy", default=20,
-               help="Amount of workers one proxy should serve to.")
-]
-
-benchmark_group = cfg.OptGroup(name="benchmark", title="benchmark options")
-CONF.register_opts(SAHARA_BENCHMARK_OPTS, group=benchmark_group)
-
 
 class SaharaScenario(scenario.OpenStackScenario):
     """Base class for Sahara scenarios with basic atomic actions."""
@@ -130,11 +107,11 @@ class SaharaScenario(scenario.OpenStackScenario):
         self.clients("sahara").node_group_templates.delete(node_group.id)
 
     def _wait_active(self, cluster_object):
-        utils.wait_for(
+        utils.wait_for_status(
             resource=cluster_object, ready_statuses=["active"],
             failure_statuses=["error"], update_resource=self._update_cluster,
-            timeout=CONF.benchmark.sahara_cluster_create_timeout,
-            check_interval=CONF.benchmark.sahara_cluster_check_interval)
+            timeout=CONF.openstack.sahara_cluster_create_timeout,
+            check_interval=CONF.openstack.sahara_cluster_check_interval)
 
     def _setup_neutron_floating_ip_pool(self, name_or_id):
         if name_or_id:
@@ -147,9 +124,10 @@ class SaharaScenario(scenario.OpenStackScenario):
                     if net["name"] == name_or_id:
                         return net["id"]
                 # If the name is not found in the list. Exit with error.
-                raise exceptions.BenchmarkSetupFailure(
-                    "Could not resolve Floating IP Pool name %s to id" %
-                    name_or_id)
+                raise exceptions.ContextSetupFailure(
+                    ctx_name=self.get_name(),
+                    msg="Could not resolve Floating IP Pool name %s to id"
+                        % name_or_id)
         else:
             # Pool is not provided. Using the one set as GW for current router.
 
@@ -321,7 +299,7 @@ class SaharaScenario(scenario.OpenStackScenario):
 
         if enable_proxy:
             proxies_count = int(
-                workers_count / CONF.benchmark.sahara_workers_per_proxy)
+                workers_count / CONF.openstack.sahara_workers_per_proxy)
         else:
             proxies_count = 0
 
@@ -430,7 +408,7 @@ class SaharaScenario(scenario.OpenStackScenario):
         scale_object required by Sahara API and waits for the scaling to
         complete.
 
-        NOTE: This method is not meant to be called directly in benchmarks.
+        NOTE: This method is not meant to be called directly in scenarios.
         There two specific scaling methods of up and down scaling which have
         different atomic timers.
         """
@@ -480,13 +458,13 @@ class SaharaScenario(scenario.OpenStackScenario):
 
         utils.wait_for(
             resource=cluster,
-            timeout=CONF.benchmark.sahara_cluster_delete_timeout,
-            check_interval=CONF.benchmark.sahara_cluster_check_interval,
+            timeout=CONF.openstack.sahara_cluster_delete_timeout,
+            check_interval=CONF.openstack.sahara_cluster_check_interval,
             is_ready=self._is_cluster_deleted)
 
     def _is_cluster_deleted(self, cluster):
-        LOG.debug("Checking cluster `%s` to be deleted. Status: `%s`" %
-                  (cluster.name, cluster.status))
+        LOG.debug("Checking cluster `%s` to be deleted. Status: `%s`"
+                  % (cluster.name, cluster.status))
         try:
             self.clients("sahara").clusters.get(cluster.id)
             return False
@@ -503,7 +481,7 @@ class SaharaScenario(scenario.OpenStackScenario):
 
         if ds_type == "swift":
             raise exceptions.RallyException(
-                _("Swift Data Sources are not implemented yet"))
+                "Swift Data Sources are not implemented yet")
 
         url = url_prefix.rstrip("/") + "/%s" % self.generate_random_name()
 
@@ -546,8 +524,8 @@ class SaharaScenario(scenario.OpenStackScenario):
             utils.wait_for(
                 resource=job_execution.id,
                 is_ready=self._job_execution_is_finished,
-                timeout=CONF.benchmark.sahara_job_execution_timeout,
-                check_interval=CONF.benchmark.sahara_job_check_interval)
+                timeout=CONF.openstack.sahara_job_execution_timeout,
+                check_interval=CONF.openstack.sahara_job_check_interval)
 
         run(self)
 
@@ -555,8 +533,8 @@ class SaharaScenario(scenario.OpenStackScenario):
         status = self.clients("sahara").job_executions.get(je_id).info[
             "status"].lower()
 
-        LOG.debug("Checking for Job Execution %s to complete. Status: %s" %
-                  (je_id, status))
+        LOG.debug("Checking for Job Execution %s to complete. Status: %s"
+                  % (je_id, status))
         if status in ("success", "succeeded"):
             return True
         elif status in ("failed", "killed"):
