@@ -21,10 +21,8 @@ import netaddr
 from oslo_config import cfg
 import six
 
-from rally.common.i18n import _
 from rally.common import logging
 from rally.common import sshutils
-from rally.plugins.openstack.scenarios.cinder import utils as cinder_utils
 from rally.plugins.openstack.scenarios.nova import utils as nova_utils
 from rally.plugins.openstack.wrappers import network as network_wrapper
 from rally.task import atomic
@@ -32,16 +30,8 @@ from rally.task import utils
 
 LOG = logging.getLogger(__name__)
 
-VM_BENCHMARK_OPTS = [
-    cfg.FloatOpt("vm_ping_poll_interval", default=1.0,
-                 help="Interval between checks when waiting for a VM to "
-                 "become pingable"),
-    cfg.FloatOpt("vm_ping_timeout", default=120.0,
-                 help="Time to wait for a VM to become pingable")]
 
 CONF = cfg.CONF
-benchmark_group = cfg.OptGroup(name="benchmark", title="benchmark options")
-CONF.register_opts(VM_BENCHMARK_OPTS, group=benchmark_group)
 
 
 class Host(object):
@@ -90,7 +80,7 @@ class Host(object):
         return not self.__eq__(other)
 
 
-class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
+class VMScenario(nova_utils.NovaScenario):
     """Base class for VM scenarios with basic atomic actions.
 
     VM scenarios are scenarios executed inside some launched VM instance.
@@ -133,8 +123,9 @@ class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
                                  "or list type")
             cmd.extend(remote_path)
             if command.get("local_path"):
-                ssh.put_file(command["local_path"], remote_path[-1],
-                             mode=self.USER_RWX_OTHERS_RX_ACCESS_MODE)
+                ssh.put_file(os.path.expanduser(
+                    command["local_path"]), remote_path[-1],
+                    mode=self.USER_RWX_OTHERS_RX_ACCESS_MODE)
 
         if command.get("script_file"):
             stdin = open(os.path.expanduser(command["script_file"]), "rb")
@@ -174,24 +165,25 @@ class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
         internal_network = list(server.networks)[0]
         fixed_ip = server.addresses[internal_network][0]["addr"]
 
-        fip = network_wrapper.wrap(self.clients, self).create_floating_ip(
-            ext_network=floating_network,
-            tenant_id=server.tenant_id, fixed_ip=fixed_ip)
+        with atomic.ActionTimer(self, "neutron.create_floating_ip"):
+            fip = network_wrapper.wrap(self.clients, self).create_floating_ip(
+                ext_network=floating_network,
+                tenant_id=server.tenant_id, fixed_ip=fixed_ip)
 
-        self._associate_floating_ip(server, fip["ip"], fixed_address=fixed_ip,
-                                    atomic_action=False)
+        self._associate_floating_ip(server, fip["ip"], fixed_address=fixed_ip)
 
         return fip
 
     @atomic.action_timer("vm.delete_floating_ip")
     def _delete_floating_ip(self, server, fip):
         with logging.ExceptionLogger(
-                LOG, _("Unable to delete IP: %s") % fip["ip"]):
+                LOG, "Unable to delete IP: %s" % fip["ip"]):
             if self.check_ip_address(fip["ip"])(server):
-                self._dissociate_floating_ip(server, fip["ip"],
-                                             atomic_action=False)
-                network_wrapper.wrap(self.clients, self).delete_floating_ip(
-                    fip["id"], wait=True)
+                self._dissociate_floating_ip(server, fip["ip"])
+                with atomic.ActionTimer(self, "neutron.delete_floating_ip"):
+                    network_wrapper.wrap(self.clients,
+                                         self).delete_floating_ip(
+                        fip["id"], wait=True)
 
     def _delete_server_with_fip(self, server, fip, force_delete=False):
         if fip["is_floating"]:
@@ -209,8 +201,8 @@ class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
             server,
             ready_statuses=[Host.ICMP_UP_STATUS],
             update_resource=Host.update_status,
-            timeout=CONF.benchmark.vm_ping_timeout,
-            check_interval=CONF.benchmark.vm_ping_poll_interval
+            timeout=CONF.openstack.vm_ping_timeout,
+            check_interval=CONF.openstack.vm_ping_poll_interval
         )
 
     def _run_command(self, server_ip, port, username, password, command,
