@@ -29,23 +29,8 @@ import six
 
 LOG = logging.getLogger(__name__)
 
-GLANCE_BENCHMARK_OPTS = [
-    cfg.FloatOpt("glance_image_create_prepoll_delay",
-                 default=2.0,
-                 help="Time to sleep after creating a resource before "
-                      "polling for it status"),
-    cfg.FloatOpt("glance_image_create_timeout",
-                 default=120.0,
-                 help="Time to wait for glance image to be created."),
-    cfg.FloatOpt("glance_image_create_poll_interval",
-                 default=1.0,
-                 help="Interval between checks when waiting for image "
-                      "creation.")
-]
 
 CONF = cfg.CONF
-benchmark_group = cfg.OptGroup(name="benchmark", title="benchmark options")
-CONF.register_opts(GLANCE_BENCHMARK_OPTS, group=benchmark_group)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -59,7 +44,6 @@ class GlanceWrapper(object):
 
         This serves to fetch the latest data on the image for the
         various wait_for_*() functions.
-
         Must raise rally.exceptions.GetResourceNotFound if the
         resource is not found or deleted.
         """
@@ -114,14 +98,14 @@ class GlanceV1Wrapper(GlanceWrapper):
 
             image = self.client.images.create(**kw)
 
-            rutils.interruptable_sleep(CONF.benchmark.
+            rutils.interruptable_sleep(CONF.openstack.
                                        glance_image_create_prepoll_delay)
 
             image = utils.wait_for_status(
                 image, ["active"],
                 update_resource=self.get_image,
-                timeout=CONF.benchmark.glance_image_create_timeout,
-                check_interval=CONF.benchmark.
+                timeout=CONF.openstack.glance_image_create_timeout,
+                check_interval=CONF.openstack.
                 glance_image_create_poll_interval)
         finally:
             if "data" in kw:
@@ -167,15 +151,15 @@ class GlanceV2Wrapper(GlanceWrapper):
 
         image = self.client.images.create(**kw)
 
-        rutils.interruptable_sleep(CONF.benchmark.
+        rutils.interruptable_sleep(CONF.openstack.
                                    glance_image_create_prepoll_delay)
 
         start = time.time()
         image = utils.wait_for_status(
             image, ["queued"],
             update_resource=self.get_image,
-            timeout=CONF.benchmark.glance_image_create_timeout,
-            check_interval=CONF.benchmark.
+            timeout=CONF.openstack.glance_image_create_timeout,
+            check_interval=CONF.openstack.
             glance_image_create_poll_interval)
         timeout = time.time() - start
 
@@ -198,7 +182,7 @@ class GlanceV2Wrapper(GlanceWrapper):
             image, ["active"],
             update_resource=self.get_image,
             timeout=timeout,
-            check_interval=CONF.benchmark.
+            check_interval=CONF.openstack.
             glance_image_create_poll_interval)
 
     def set_visibility(self, image, visibility="public"):
@@ -210,6 +194,11 @@ class GlanceV2Wrapper(GlanceWrapper):
 
 def wrap(client, owner):
     """Returns glanceclient wrapper based on glance client version."""
+    LOG.warning("Method wrap from %s and whole Glance wrappers are "
+                "deprecated since Rally 0.10.0 and will be removed soon. Use "
+                "rally.plugins.openstack.services.image.image.Image "
+                "instead." % __file__)
+
     version = client.choose_version()
     if version == "1":
         return GlanceV1Wrapper(client(), owner)
