@@ -26,14 +26,6 @@ import tempfile
 from oslo_utils import encodeutils
 from six.moves import configparser
 
-
-TEST_ENV = {
-    "OS_USERNAME": "admin",
-    "OS_PASSWORD": "admin",
-    "OS_TENANT_NAME": "admin",
-    "OS_AUTH_URL": "http://fake/",
-}
-
 DEPLOYMENT_FILE = "/tmp/rally_functests_main_deployment.json"
 
 
@@ -83,7 +75,8 @@ class Rally(object):
 
     def __init__(self, fake=False, force_new_db=False):
         if not os.path.exists(DEPLOYMENT_FILE):
-            subprocess.call(["rally", "deployment", "config"],
+            subprocess.call(["rally", "--log-file", "/dev/null",
+                             "deployment", "config"],
                             stdout=open(DEPLOYMENT_FILE, "w"))
 
         # NOTE(sskripnick): we should change home dir to avoid races
@@ -92,6 +85,20 @@ class Rally(object):
         self.env = copy.deepcopy(os.environ)
         self.env["HOME"] = self.tmp_dir
         self.config_filename = None
+        self.method_name = None
+        self.class_name = None
+
+        caller_frame = inspect.currentframe().f_back
+        if caller_frame.f_code.co_name == "__call__":
+            caller_frame = caller_frame.f_back
+
+        self.method_name = caller_frame.f_code.co_name
+        if self.method_name == "setUp":
+            raise Exception("No rally instance should be generated in "
+                            "setUp method")
+
+        test_object = caller_frame.f_locals["self"]
+        self.class_name = test_object.__class__.__name__
 
         if force_new_db or ("RCI_KEEP_DB" not in os.environ):
             config_filename = os.path.join(self.tmp_dir, "conf")
@@ -102,17 +109,16 @@ class Rally(object):
             with open(config_filename, "w") as conf:
                 config.write(conf)
             self.args = ["rally", "--config-file", config_filename]
-            subprocess.call(["rally-manage", "--config-file", config_filename,
+            subprocess.call(["rally", "--config-file", config_filename,
                              "db", "recreate"], env=self.env)
             self.config_filename = config_filename
         else:
             self.args = ["rally"]
-            subprocess.call(["rally-manage", "db", "recreate"], env=self.env)
+            subprocess.call(["rally", "db", "recreate"], env=self.env)
 
         self.reports_root = os.environ.get("REPORTS_ROOT",
                                            "rally-cli-output-files")
         self._created_files = []
-
         self("deployment create --file %s --name MAIN" % DEPLOYMENT_FILE,
              write_report=False)
 
@@ -139,20 +145,13 @@ class Rally(object):
 
         :return: complete report name to write report
         """
-        caller_frame = inspect.currentframe().f_back
-        if caller_frame.f_code.co_name == "__call__":
-            caller_frame = caller_frame.f_back
 
-        method_name = caller_frame.f_code.co_name
-        test_object = caller_frame.f_locals["self"]
-        class_name = test_object.__class__.__name__
-
-        self._safe_make_dirs("%s/%s" % (self.reports_root, class_name))
+        self._safe_make_dirs("%s/%s" % (self.reports_root, self.class_name))
 
         suff = suffix or ""
         ext = extension or "txt"
-        path = "%s/%s/%s%s.%s" % (self.reports_root, class_name,
-                                  method_name, suff, ext)
+        path = "%s/%s/%s%s.%s" % (self.reports_root, self.class_name,
+                                  self.method_name, suff, ext)
 
         if path not in self._created_files:
             if os.path.exists(path):
@@ -190,24 +189,32 @@ class Rally(object):
         if not isinstance(cmd, list):
             cmd = cmd.split(" ")
         try:
-            output = encodeutils.safe_decode(subprocess.check_output(
-                self.args + cmd, stderr=subprocess.STDOUT, env=self.env))
+            if getjson:
+                cmd = self.args + ["--log-file", "/dev/null"] + cmd
+            else:
+                cmd = self.args + cmd
 
+            output = encodeutils.safe_decode(subprocess.check_output(
+                cmd, stderr=subprocess.STDOUT, env=self.env))
+
+            if getjson:
+                return json.loads(output)
+
+            return output
+
+        except subprocess.CalledProcessError as e:
+            output = e.output
+            raise RallyCliError(cmd, e.returncode, e.output)
+        finally:
             if write_report:
                 if not report_path:
                     report_path = self.gen_report_path(
                         suffix=suffix, extension=extension, keep_old=keep_old)
                 with open(report_path, "a") as rep:
                     if not raw:
-                        rep.write("\n%s:\n" % " ".join(self.args + cmd))
+                        rep.write("\n%s:\n" % " ".join(cmd))
                     rep.write("%s\n" % output)
 
-            if getjson:
-                return json.loads(output)
-            return output
-        except subprocess.CalledProcessError as e:
-            raise RallyCliError(cmd, e.returncode, e.output)
-
 
 def get_global(global_key, env):
     home_dir = env.get("HOME")
