@@ -15,16 +15,225 @@
 
 import mock
 
-from rally.common import objects
 from rally import consts
 from rally import exceptions
 from rally.plugins.openstack.context.keystone import users
+from rally.plugins.openstack import credential as oscredential
 from tests.unit import test
 
 CTX = "rally.plugins.openstack.context.keystone.users"
 
 
-class UserGeneratorTestCase(test.ScenarioTestCase):
+class UserGeneratorBaseTestCase(test.ScenarioTestCase):
+    def setUp(self):
+        super(UserGeneratorBaseTestCase, self).setUp()
+        self.osclients_patcher = mock.patch("%s.osclients" % CTX)
+        self.osclients = self.osclients_patcher.start()
+        self.addCleanup(self.osclients_patcher.stop)
+
+        self.deployment_patcher = mock.patch("%s.objects.Deployment.get" % CTX)
+        self.deployment_get = self.deployment_patcher.start()
+        self.addCleanup(self.deployment_patcher.stop)
+
+        self.deployment_uuid = "deployment_id"
+        self.admin_cred = mock.MagicMock()
+
+        self.context.update({
+            "config": {"users": {}},
+            "admin": {"credential": self.admin_cred},
+            "users": [],
+            "task": {"uuid": "task_id",
+                     "deployment_uuid": self.deployment_uuid}
+        })
+
+    def test___init__for_new_users(self):
+        deployment = self.deployment_get.return_value
+        deployment.get_credentials_for.return_value = {"users": []}
+        self.context["config"]["users"] = {
+            "tenants": 1, "users_per_tenant": 1,
+            "resource_management_workers": 1}
+
+        user_generator = users.UserGenerator(self.context)
+
+        self.assertEqual([], user_generator.existing_users)
+        self.assertEqual(self.admin_cred.project_domain_name,
+                         user_generator.config["project_domain"])
+        self.assertEqual(self.admin_cred.user_domain_name,
+                         user_generator.config["user_domain"])
+
+        self.deployment_get.assert_called_once_with(self.deployment_uuid)
+        deployment.get_credentials_for.assert_called_once_with("openstack")
+
+        self.deployment_get.reset_mock()
+        deployment.get_credentials_for.reset_mock()
+
+        # the case #2 - existing users are presented in deployment but
+        #   the user forces to create new ones
+        deployment.get_credentials_for.return_value = {"users": [mock.Mock()]}
+
+        user_generator = users.UserGenerator(self.context)
+
+        self.assertEqual([], user_generator.existing_users)
+        self.assertEqual(self.admin_cred.project_domain_name,
+                         user_generator.config["project_domain"])
+        self.assertEqual(self.admin_cred.user_domain_name,
+                         user_generator.config["user_domain"])
+
+        self.deployment_get.assert_called_once_with(self.deployment_uuid)
+        deployment.get_credentials_for.assert_called_once_with("openstack")
+
+    def test___init__for_existing_users(self):
+        deployment = self.deployment_get.return_value
+        foo_user = mock.Mock()
+        deployment.get_credentials_for.return_value = {"users": [foo_user]}
+
+        user_generator = users.UserGenerator(self.context)
+
+        self.assertEqual([foo_user], user_generator.existing_users)
+        self.assertEqual({"user_choice_method": "random"},
+                         user_generator.config)
+
+        self.deployment_get.assert_called_once_with(self.deployment_uuid)
+        deployment.get_credentials_for.assert_called_once_with("openstack")
+
+        self.deployment_get.reset_mock()
+        deployment.get_credentials_for.reset_mock()
+
+        # the case #2: the config with `user_choice_method` option
+        self.context["config"]["users"] = {"user_choice_method": "foo"}
+
+        user_generator = users.UserGenerator(self.context)
+
+        self.assertEqual([foo_user], user_generator.existing_users)
+        self.assertEqual({"user_choice_method": "foo"}, user_generator.config)
+
+        self.deployment_get.assert_called_once_with(self.deployment_uuid)
+        deployment.get_credentials_for.assert_called_once_with("openstack")
+
+    def test_setup(self):
+        user_generator = users.UserGenerator(self.context)
+        user_generator.use_existing_users = mock.Mock()
+        user_generator.create_users = mock.Mock()
+
+        # no existing users -> new users should be created
+        user_generator.existing_users = []
+
+        user_generator.setup()
+
+        user_generator.create_users.assert_called_once_with()
+        self.assertFalse(user_generator.use_existing_users.called)
+
+        user_generator.create_users.reset_mock()
+        user_generator.use_existing_users.reset_mock()
+
+        # existing_users is not empty -> existing users should be created
+        user_generator.existing_users = [mock.Mock()]
+
+        user_generator.setup()
+
+        user_generator.use_existing_users.assert_called_once_with()
+        self.assertFalse(user_generator.create_users.called)
+
+    def test_cleanup(self):
+        user_generator = users.UserGenerator(self.context)
+        user_generator._remove_default_security_group = mock.Mock()
+        user_generator._delete_users = mock.Mock()
+        user_generator._delete_tenants = mock.Mock()
+
+        # In case if existing users nothing should be done
+        user_generator.existing_users = [mock.Mock]
+
+        user_generator.cleanup()
+
+        self.assertFalse(user_generator._remove_default_security_group.called)
+        self.assertFalse(user_generator._delete_users.called)
+        self.assertFalse(user_generator._delete_tenants.called)
+
+        # In case when new users were created, the proper cleanup should be
+        #   performed
+        user_generator.existing_users = []
+
+        user_generator.cleanup()
+
+        user_generator._remove_default_security_group.assert_called_once_with()
+        user_generator._delete_users.assert_called_once_with()
+        user_generator._delete_tenants.assert_called_once_with()
+
+
+class UserGeneratorForExistingUsersTestCase(test.ScenarioTestCase):
+    def setUp(self):
+        super(UserGeneratorForExistingUsersTestCase, self).setUp()
+        self.osclients_patcher = mock.patch("%s.osclients" % CTX)
+        self.osclients = self.osclients_patcher.start()
+        self.addCleanup(self.osclients_patcher.stop)
+
+        self.deployment_patcher = mock.patch("%s.objects.Deployment.get" % CTX)
+        self.deployment_get = self.deployment_patcher.start()
+        self.addCleanup(self.deployment_patcher.stop)
+
+        self.deployment_uuid = "deployment_id"
+
+        self.context.update({
+            "config": {"users": {}},
+            "users": [],
+            "task": {"uuid": "task_id",
+                     "deployment_uuid": self.deployment_uuid}
+        })
+
+    def test_use_existing_users(self):
+        user1 = mock.MagicMock(tenant_id="1", user_id="1",
+                               tenant_name="proj", username="usr")
+        user2 = mock.MagicMock(tenant_id="1", user_id="2",
+                               tenant_name="proj", username="usr")
+        user3 = mock.MagicMock(tenant_id="2", user_id="3",
+                               tenant_name="proj", username="usr")
+
+        user_list = [user1, user2, user3]
+
+        class AuthRef(object):
+            USER_ID_COUNT = 0
+            PROJECT_ID_COUNT = 0
+
+            @property
+            def user_id(self):
+                self.USER_ID_COUNT += 1
+                return user_list[self.USER_ID_COUNT - 1].user_id
+
+            @property
+            def project_id(self):
+                self.PROJECT_ID_COUNT += 1
+                return user_list[self.PROJECT_ID_COUNT - 1].tenant_id
+
+        auth_ref = AuthRef()
+
+        user1.clients.return_value.keystone.auth_ref = auth_ref
+        user2.clients.return_value.keystone.auth_ref = auth_ref
+        user3.clients.return_value.keystone.auth_ref = auth_ref
+
+        deployment = self.deployment_get.return_value
+        deployment.get_credentials_for.return_value = {"users": user_list}
+
+        user_generator = users.UserGenerator(self.context)
+        user_generator.setup()
+
+        self.assertIn("users", self.context)
+        self.assertIn("tenants", self.context)
+        self.assertIn("user_choice_method", self.context)
+        self.assertEqual("random", self.context["user_choice_method"])
+        self.assertEqual(
+            [{"id": user1.user_id, "credential": user1,
+              "tenant_id": user1.tenant_id},
+             {"id": user2.user_id, "credential": user2,
+              "tenant_id": user2.tenant_id},
+             {"id": user3.user_id, "credential": user3,
+              "tenant_id": user3.tenant_id}], self.context["users"]
+        )
+        self.assertEqual({"1": {"id": "1", "name": user1.tenant_name},
+                          "2": {"id": "2", "name": user3.tenant_name}},
+                         self.context["tenants"])
+
+
+class UserGeneratorForNewUsersTestCase(test.ScenarioTestCase):
 
     tenants_num = 1
     users_per_tenant = 5
@@ -32,9 +241,19 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
     threads = 10
 
     def setUp(self):
-        super(UserGeneratorTestCase, self).setUp()
+        super(UserGeneratorForNewUsersTestCase, self).setUp()
         self.osclients_patcher = mock.patch("%s.osclients" % CTX)
         self.osclients = self.osclients_patcher.start()
+        self.addCleanup(self.osclients_patcher.stop)
+
+        self.deployment_patcher = mock.patch("%s.objects.Deployment.get" % CTX)
+        self.deployment_get = self.deployment_patcher.start()
+        self.addCleanup(self.deployment_patcher.stop)
+
+        # Force the case of creating new users
+        deployment = self.deployment_get.return_value
+        deployment.get_credentials_for.return_value = {"users": []}
+
         self.context.update({
             "config": {
                 "users": {
@@ -45,13 +264,9 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
             },
             "admin": {"credential": mock.MagicMock()},
             "users": [],
-            "task": {"uuid": "task_id"}
+            "task": {"uuid": "task_id", "deployment_uuid": "dep_uuid"}
         })
 
-    def tearDown(self):
-        self.osclients_patcher.stop()
-        super(UserGeneratorTestCase, self).tearDown()
-
     @mock.patch("%s.network.wrap" % CTX)
     def test__remove_default_security_group_not_needed(self, mock_wrap):
         services = {"compute": consts.Service.NOVA}
@@ -98,7 +313,15 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         admin_clients.services.return_value = {
             "compute": consts.Service.NOVA,
             "neutron": consts.Service.NEUTRON}
-        user_clients = [mock.Mock(), mock.Mock()]
+        user1 = mock.Mock()
+        user1.neutron.return_value.list_security_groups.return_value = {
+            "security_groups": [{"id": "id-1", "name": "default"},
+                                {"id": "id-2", "name": "not-default"}]}
+        user2 = mock.Mock()
+        user2.neutron.return_value.list_security_groups.return_value = {
+            "security_groups": [{"id": "id-3", "name": "default"},
+                                {"id": "id-4", "name": "not-default"}]}
+        user_clients = [user1, user2]
         self.osclients.Clients.side_effect = [admin_clients] + user_clients
 
         mock_iterate_per_tenants.return_value = [
@@ -117,69 +340,14 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
             for u, t in mock_iterate_per_tenants.return_value]
         self.osclients.Clients.assert_has_calls(expected, any_order=True)
 
-        expected_deletes = []
-        for clients in user_clients:
-            user_nova = clients.nova.return_value
-            user_nova.security_groups.find.assert_called_once_with(
-                name="default")
-            expected_deletes.append(
-                mock.call(user_nova.security_groups.find.return_value.id))
-
-        nova_admin = admin_clients.neutron.return_value
-        nova_admin.delete_security_group.assert_has_calls(expected_deletes,
-                                                          any_order=True)
-
-    @mock.patch("rally.task.utils.check_service_status",
-                return_value=True)
-    def test__remove_associated_networks(self, mock_check_service_status):
-        def fake_get_network(req_network):
-            for network in networks:
-                if network.project_id == req_network.project_id:
-                    return network
-
-        networks = [mock.MagicMock(project_id="t1"),
-                    mock.MagicMock(project_id="t4")]
-        nova_admin = mock.MagicMock()
-        clients = mock.MagicMock()
-        self.osclients.Clients.return_value = clients
-        clients.services.return_value = {"compute": "nova"}
-        clients.nova.return_value = nova_admin
-        nova_admin.networks.list.return_value = networks
-        nova_admin.networks.get = fake_get_network
-        user_generator = users.UserGenerator(self.context)
-        user_generator.context["tenants"] = {"t1": {"id": "t1", "name": "t1"},
-                                             "t2": {"id": "t2", "name": "t2"}}
-        user_generator._remove_associated_networks()
-        mock_check_service_status.assert_called_once_with(mock.ANY,
-                                                          "nova-network")
-        nova_admin.networks.disassociate.assert_called_once_with(networks[0])
-
-    @mock.patch("rally.task.utils.check_service_status",
-                return_value=True)
-    def test__remove_associated_networks_failure(self,
-                                                 mock_check_service_status):
-        def fake_get_network(req_network):
-            for network in networks:
-                if network.project_id == req_network.project_id:
-                    return network
-
-        networks = [mock.MagicMock(project_id="t1"),
-                    mock.MagicMock(project_id="t4")]
-        nova_admin = mock.MagicMock()
-        clients = mock.MagicMock()
-        self.osclients.Clients.return_value = clients
-        clients.services.return_value = {"compute": "nova"}
-        clients.nova.return_value = nova_admin
-        nova_admin.networks.list.return_value = networks
-        nova_admin.networks.get = fake_get_network
-        nova_admin.networks.disassociate.side_effect = Exception()
-        user_generator = users.UserGenerator(self.context)
-        user_generator.context["tenants"] = {"t1": {"id": "t1", "name": "t1"},
-                                             "t2": {"id": "t2", "name": "t2"}}
-        user_generator._remove_associated_networks()
-        mock_check_service_status.assert_called_once_with(mock.ANY,
-                                                          "nova-network")
-        nova_admin.networks.disassociate.assert_called_once_with(networks[0])
+        user_net = user1.neutron.return_value
+        user_net.list_security_groups.assert_called_once_with()
+        user_net = user2.neutron.return_value
+        user_net.list_security_groups.assert_called_once_with()
+        admin_neutron = admin_clients.neutron.return_value
+        self.assertEqual(
+            [mock.call("id-1"), mock.call("id-3")],
+            admin_neutron.delete_security_group.call_args_list)
 
     @mock.patch("%s.identity" % CTX)
     def test__create_tenants(self, mock_identity):
@@ -208,7 +376,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         user_generator.context["tenants"] = {"t1": {"id": "t1", "name": "t1"},
                                              "t2": {"id": "t2", "name": "t2"}}
         user_generator._delete_tenants()
-        self.assertEqual(len(user_generator.context["tenants"]), 0)
+        self.assertEqual(0, len(user_generator.context["tenants"]))
 
     @mock.patch("%s.identity" % CTX)
     def test__delete_tenants_failure(self, mock_identity):
@@ -218,7 +386,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         user_generator.context["tenants"] = {"t1": {"id": "t1", "name": "t1"},
                                              "t2": {"id": "t2", "name": "t2"}}
         user_generator._delete_tenants()
-        self.assertEqual(len(user_generator.context["tenants"]), 0)
+        self.assertEqual(0, len(user_generator.context["tenants"]))
 
     @mock.patch("%s.identity" % CTX)
     def test__delete_users(self, mock_identity):
@@ -227,7 +395,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         user2 = mock.MagicMock()
         user_generator.context["users"] = [user1, user2]
         user_generator._delete_users()
-        self.assertEqual(len(user_generator.context["users"]), 0)
+        self.assertEqual(0, len(user_generator.context["users"]))
 
     @mock.patch("%s.identity" % CTX)
     def test__delete_users_failure(self, mock_identity):
@@ -238,7 +406,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         user2 = mock.MagicMock()
         user_generator.context["users"] = [user1, user2]
         user_generator._delete_users()
-        self.assertEqual(len(user_generator.context["users"]), 0)
+        self.assertEqual(0, len(user_generator.context["users"]))
 
     @mock.patch("%s.identity" % CTX)
     def test_setup_and_cleanup(self, mock_identity):
@@ -246,15 +414,16 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
 
             ctx.setup()
 
-            self.assertEqual(len(ctx.context["users"]),
-                             self.users_num)
-            self.assertEqual(len(ctx.context["tenants"]),
-                             self.tenants_num)
+            self.assertEqual(self.users_num,
+                             len(ctx.context["users"]))
+            self.assertEqual(self.tenants_num,
+                             len(ctx.context["tenants"]))
+
             self.assertEqual("random", ctx.context["user_choice_method"])
 
         # Cleanup (called by content manager)
-        self.assertEqual(len(ctx.context["users"]), 0)
-        self.assertEqual(len(ctx.context["tenants"]), 0)
+        self.assertEqual(0, len(ctx.context["users"]))
+        self.assertEqual(0, len(ctx.context["tenants"]))
 
     @mock.patch("rally.common.broker.LOG.warning")
     @mock.patch("%s.identity" % CTX)
@@ -274,16 +443,17 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
     def test_users_and_tenants_in_context(self, mock_identity):
         identity_service = mock_identity.Identity.return_value
 
-        credential = objects.Credential("foo_url", "foo", "foo_pass",
-                                        https_insecure=True,
-                                        https_cacert="cacert")
+        credential = oscredential.OpenStackCredential(
+            "foo_url", "foo", "foo_pass",
+            https_insecure=True,
+            https_cacert="cacert")
         tmp_context = dict(self.context)
         tmp_context["config"]["users"] = {"tenants": 1,
                                           "users_per_tenant": 2,
                                           "resource_management_workers": 1}
         tmp_context["admin"]["credential"] = credential
 
-        credential_dict = credential.to_dict(False)
+        credential_dict = credential.to_dict()
         user_list = [mock.MagicMock(id="id_%d" % i)
                      for i in range(self.users_num)]
         identity_service.create_user.side_effect = user_list
@@ -302,7 +472,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                 self.assertEqual(set(["id", "credential", "tenant_id"]),
                                  set(user.keys()))
 
-                user_credential_dict = user["credential"].to_dict(False)
+                user_credential_dict = user["credential"].to_dict()
 
                 excluded_keys = ["auth_url", "username", "password",
                                  "tenant_name", "region_name",
@@ -318,12 +488,12 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
 
             for (user, tenant_id, orig_user) in zip(ctx.context["users"],
                                                     tenants_ids, user_list):
-                self.assertEqual(user["id"], orig_user.id)
-                self.assertEqual(user["tenant_id"], tenant_id)
+                self.assertEqual(orig_user.id, user["id"])
+                self.assertEqual(tenant_id, user["tenant_id"])
 
     @mock.patch("%s.identity" % CTX)
     def test_users_contains_correct_endpoint_type(self, mock_identity):
-        credential = objects.Credential(
+        credential = oscredential.OpenStackCredential(
             "foo_url", "foo", "foo_pass",
             endpoint_type=consts.EndpointType.INTERNAL)
         config = {
@@ -335,7 +505,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                 }
             },
             "admin": {"credential": credential},
-            "task": {"uuid": "task_id"}
+            "task": {"uuid": "task_id", "deployment_uuid": "deployment_id"}
         }
 
         user_generator = users.UserGenerator(config)
@@ -346,7 +516,8 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
 
     @mock.patch("%s.identity" % CTX)
     def test_users_contains_default_endpoint_type(self, mock_identity):
-        credential = objects.Credential("foo_url", "foo", "foo_pass")
+        credential = oscredential.OpenStackCredential(
+            "foo_url", "foo", "foo_pass")
         config = {
             "config": {
                 "users": {
@@ -356,7 +527,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                 }
             },
             "admin": {"credential": credential},
-            "task": {"uuid": "task_id"}
+            "task": {"uuid": "task_id", "deployment_uuid": "deployment_id"}
         }
 
         user_generator = users.UserGenerator(config)
