@@ -27,7 +27,7 @@ CTX = "rally.plugins.openstack.context.sahara"
 
 class SaharaClusterTestCase(test.ScenarioTestCase):
 
-    patch_benchmark_utils = False
+    patch_task_utils = False
 
     def setUp(self):
         super(SaharaClusterTestCase, self).setUp()
@@ -46,8 +46,7 @@ class SaharaClusterTestCase(test.ScenarioTestCase):
                                        "tenant_id": str(i),
                                        "credential": mock.MagicMock()})
 
-        CONF.set_override("sahara_cluster_check_interval", 0, "benchmark",
-                          enforce_type=True)
+        CONF.set_override("sahara_cluster_check_interval", 0, "openstack")
 
         self.context.update({
             "config": {
@@ -111,7 +110,7 @@ class SaharaClusterTestCase(test.ScenarioTestCase):
             names=["sahara.clusters"],
             users=self.context["users"],
             superclass=sahara_utils.SaharaScenario,
-            task_id=self.context["task"]["uuid"])
+            task_id=self.context["owner_id"])
 
     @mock.patch("%s.sahara_cluster.utils.SaharaScenario._launch_cluster" % CTX,
                 return_value=mock.MagicMock(id=42))
@@ -146,4 +145,4 @@ class SaharaClusterTestCase(test.ScenarioTestCase):
             mock.MagicMock(status="error")
         ]
 
-        self.assertRaises(exceptions.SaharaClusterFailure, sahara_ctx.setup)
+        self.assertRaises(exceptions.ContextSetupFailure, sahara_ctx.setup)
