@@ -17,7 +17,7 @@ import mock
 from oslo_config import cfg
 
 from rally import exceptions
-from rally import osclients
+from rally.plugins.openstack import osclients
 from rally.plugins.openstack.scenarios.cinder import utils
 from tests.unit import fakes
 from tests.unit import test
@@ -26,6 +26,35 @@ CINDER_UTILS = "rally.plugins.openstack.scenarios.cinder.utils"
 CONF = cfg.CONF
 
 
+class CinderBasicTestCase(test.ScenarioTestCase):
+
+    def _get_context(self):
+        context = test.get_test_context()
+        context.update({
+            "admin": {
+                "id": "fake_user_id",
+                "credential": mock.MagicMock()
+            },
+            "user": {"id": "fake_user_id",
+                     "credential": mock.MagicMock()},
+            "tenant": {"id": "fake", "name": "fake",
+                       "volumes": [{"id": "uuid", "size": 1}],
+                       "servers": [1]}})
+        return context
+
+    def setUp(self):
+        super(CinderBasicTestCase, self).setUp()
+
+    @mock.patch("random.choice")
+    def test_get_random_server(self, mock_choice):
+        basic = utils.CinderBasic(self._get_context())
+        server_id = mock_choice(basic.context["tenant"]["servers"])
+        return_server = basic.get_random_server()
+        basic.clients("nova").servers.get.assert_called_once_with(server_id)
+        self.assertEqual(basic.clients("nova").servers.get.return_value,
+                         return_server)
+
+
 class CinderScenarioTestCase(test.ScenarioTestCase):
 
     def setUp(self):
@@ -80,11 +109,11 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
 
         self.scenario._set_metadata(volume, sets=2, set_size=4)
         calls = self.clients("cinder").volumes.set_metadata.call_args_list
-        self.assertEqual(len(calls), 2)
+        self.assertEqual(2, len(calls))
         for call in calls:
             call_volume, metadata = call[0]
-            self.assertEqual(call_volume, volume)
-            self.assertEqual(len(metadata), 4)
+            self.assertEqual(volume, call_volume)
+            self.assertEqual(4, len(metadata))
 
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.set_4_metadatas_2_times")
@@ -95,12 +124,12 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
         keys = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l"]
         self.scenario._delete_metadata(volume, keys, deletes=3, delete_size=4)
         calls = self.clients("cinder").volumes.delete_metadata.call_args_list
-        self.assertEqual(len(calls), 3)
+        self.assertEqual(3, len(calls))
         all_deleted = []
         for call in calls:
             call_volume, del_keys = call[0]
-            self.assertEqual(call_volume, volume)
-            self.assertEqual(len(del_keys), 4)
+            self.assertEqual(volume, call_volume)
+            self.assertEqual(4, len(del_keys))
             for key in del_keys:
                 self.assertIn(key, keys)
                 self.assertNotIn(key, all_deleted)
@@ -119,15 +148,16 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
 
     def test__create_volume(self):
         return_volume = self.scenario._create_volume(1)
-        self.mock_wait_for.mock.assert_called_once_with(
+        self.mock_wait_for_status.mock.assert_called_once_with(
             self.mock_wrap.return_value.create_volume.return_value,
             ready_statuses=["available"],
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         self.mock_get_from_manager.mock.assert_called_once_with()
-        self.assertEqual(self.mock_wait_for.mock.return_value, return_volume)
+        self.assertEqual(self.mock_wait_for_status.mock.return_value,
+                         return_volume)
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.create_volume")
 
@@ -142,15 +172,16 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
         self.mock_wrap.return_value.create_volume.assert_called_once_with(
             3, display_name="TestVolume")
 
-        self.mock_wait_for.mock.assert_called_once_with(
+        self.mock_wait_for_status.mock.assert_called_once_with(
             self.mock_wrap.return_value.create_volume.return_value,
             ready_statuses=["available"],
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         self.mock_get_from_manager.mock.assert_called_once_with()
-        self.assertEqual(self.mock_wait_for.mock.return_value, return_volume)
+        self.assertEqual(self.mock_wait_for_status.mock.return_value,
+                         return_volume)
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.create_volume")
 
@@ -185,8 +216,8 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
             ready_statuses=["deleted"],
             check_deletion=True,
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=cfg.CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=cfg.CONF.benchmark
+            timeout=cfg.CONF.openstack.cinder_volume_create_timeout,
+            check_interval=cfg.CONF.openstack
             .cinder_volume_create_poll_interval)
         self.mock_get_from_manager.mock.assert_called_once_with()
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
@@ -201,12 +232,12 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
         self.scenario._extend_volume(volume, new_size={"min": 1, "max": 5})
 
         volume.extend.assert_called_once_with(volume, 3)
-        self.mock_wait_for.mock.assert_called_once_with(
+        self.mock_wait_for_status.mock.assert_called_once_with(
             volume,
             ready_statuses=["available"],
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         self.mock_get_from_manager.mock.assert_called_once_with()
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
@@ -216,12 +247,12 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
         volume = mock.Mock()
         self.clients("cinder").volumes.extend.return_value = volume
         self.scenario._extend_volume(volume, 2)
-        self.mock_wait_for.mock.assert_called_once_with(
+        self.mock_wait_for_status.mock.assert_called_once_with(
             volume,
             ready_statuses=["available"],
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=CONF.benchmark.cinder_volume_create_poll_interval
+            timeout=CONF.openstack.cinder_volume_create_timeout,
+            check_interval=CONF.openstack.cinder_volume_create_poll_interval
         )
         self.mock_get_from_manager.mock.assert_called_once_with()
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
@@ -241,20 +272,20 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
 
         volume.upload_to_image.assert_called_once_with(False, "test_vol",
                                                        "container", "disk")
-        self.mock_wait_for.mock.assert_has_calls([
+        self.mock_wait_for_status.mock.assert_has_calls([
             mock.call(
                 volume,
                 ready_statuses=["available"],
                 update_resource=self.mock_get_from_manager.mock.return_value,
-                timeout=CONF.benchmark.cinder_volume_create_timeout,
-                check_interval=CONF.benchmark.
+                timeout=CONF.openstack.cinder_volume_create_timeout,
+                check_interval=CONF.openstack.
                 cinder_volume_create_poll_interval),
             mock.call(
                 self.clients("glance").images.get.return_value,
                 ready_statuses=["active"],
                 update_resource=mock_wrap.return_value.get_image,
-                timeout=CONF.benchmark.glance_image_create_timeout,
-                check_interval=CONF.benchmark.
+                timeout=CONF.openstack.glance_image_create_timeout,
+                check_interval=CONF.openstack.
                 glance_image_create_poll_interval)
         ])
         self.mock_get_from_manager.mock.assert_called_once_with()
@@ -263,15 +294,16 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
     def test__create_snapshot(self):
         return_snapshot = self.scenario._create_snapshot("uuid", False)
 
-        self.mock_wait_for.mock.assert_called_once_with(
+        self.mock_wait_for_status.mock.assert_called_once_with(
             self.mock_wrap.return_value.create_snapshot.return_value,
             ready_statuses=["available"],
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=cfg.CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=cfg.CONF.benchmark
+            timeout=cfg.CONF.openstack.cinder_volume_create_timeout,
+            check_interval=cfg.CONF.openstack
             .cinder_volume_create_poll_interval)
         self.mock_get_from_manager.mock.assert_called_once_with()
-        self.assertEqual(self.mock_wait_for.mock.return_value, return_snapshot)
+        self.assertEqual(self.mock_wait_for_status.mock.return_value,
+                         return_snapshot)
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.create_snapshot")
 
@@ -284,8 +316,8 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
             ready_statuses=["deleted"],
             check_deletion=True,
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=cfg.CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=cfg.CONF.benchmark
+            timeout=cfg.CONF.openstack.cinder_volume_create_timeout,
+            check_interval=cfg.CONF.openstack
             .cinder_volume_create_poll_interval)
         self.mock_get_from_manager.mock.assert_called_once_with()
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
@@ -294,15 +326,16 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
     def test__create_backup(self):
         return_backup = self.scenario._create_backup("uuid")
 
-        self.mock_wait_for.mock.assert_called_once_with(
+        self.mock_wait_for_status.mock.assert_called_once_with(
             self.clients("cinder").backups.create.return_value,
             ready_statuses=["available"],
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=cfg.CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=cfg.CONF.benchmark
+            timeout=cfg.CONF.openstack.cinder_volume_create_timeout,
+            check_interval=cfg.CONF.openstack
             .cinder_volume_create_poll_interval)
         self.mock_get_from_manager.mock.assert_called_once_with()
-        self.assertEqual(self.mock_wait_for.mock.return_value, return_backup)
+        self.assertEqual(self.mock_wait_for_status.mock.return_value,
+                         return_backup)
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.create_backup")
 
@@ -315,8 +348,8 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
             ready_statuses=["deleted"],
             check_deletion=True,
             update_resource=self.mock_get_from_manager.mock.return_value,
-            timeout=cfg.CONF.benchmark.cinder_volume_create_timeout,
-            check_interval=cfg.CONF.benchmark
+            timeout=cfg.CONF.openstack.cinder_volume_create_timeout,
+            check_interval=cfg.CONF.openstack
             .cinder_volume_create_poll_interval)
         self.mock_get_from_manager.mock.assert_called_once_with()
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
@@ -324,7 +357,7 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
 
     def test__restore_backup(self):
         # NOTE(mdovgal): added for pep8 visual indent test passing
-        bench_cfg = cfg.CONF.benchmark
+        bench_cfg = cfg.CONF.openstack
 
         backup = mock.Mock()
         restore = mock.Mock()
@@ -333,7 +366,7 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
         self.clients("cinder").volumes.get.return_value = restore
 
         return_restore = self.scenario._restore_backup(backup.id, None)
-        self.mock_wait_for.mock.assert_has_calls([
+        self.mock_wait_for_status.mock.assert_has_calls([
             mock.call(
                 backup,
                 ready_statuses=["available"],
@@ -350,7 +383,8 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
 
         self.mock_get_from_manager.mock.assert_has_calls([mock.call(),
                                                           mock.call()])
-        self.assertEqual(self.mock_wait_for.mock.return_value, return_restore)
+        self.assertEqual(self.mock_wait_for_status.mock.return_value,
+                         return_restore)
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.restore_backup")
 
@@ -396,7 +430,7 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
     def test__delete_encryption_type(self):
         volume_type = mock.Mock()
 
-        self.assertRaises(exceptions.EncryptionTypeDeleteException,
+        self.assertRaises(exceptions.RallyException,
                           self.scenario._delete_encryption_type,
                           volume_type)
 
@@ -427,6 +461,18 @@ class CinderScenarioTestCase(test.ScenarioTestCase):
         self._test_atomic_action_timer(self.scenario.atomic_actions(),
                                        "cinder.list_encryption_type")
 
+    def test__get_volume_type(self):
+        volume_type = mock.Mock()
+        result = self.scenario._get_volume_type(volume_type)
+        self.assertEqual(
+            self.admin_clients("cinder").volume_types.get.return_value,
+            result)
+
+        self.admin_clients("cinder").volume_types.get.assert_called_once_with(
+            volume_type)
+        self._test_atomic_action_timer(self.scenario.atomic_actions(),
+                                       "cinder.get_volume_type")
+
     def test__delete_volume_type(self):
         volume_type = mock.Mock()
         self.scenario._delete_volume_type(volume_type)
