@@ -71,8 +71,7 @@ class MuranoScenarioTestCase(test.ScenarioTestCase):
         scenario = utils.MuranoScenario(context=self.context)
 
         create_app = scenario._create_service(mock_env, mock_sess,
-                                              "fake_full_name",
-                                              atomic_action=True)
+                                              "fake_full_name")
 
         self.assertEqual("app", create_app)
         self._test_atomic_action_timer(scenario.atomic_actions(),
@@ -89,8 +88,8 @@ class MuranoScenarioTestCase(test.ScenarioTestCase):
             environment.id, session.id
         )
 
-        config = CONF.benchmark
-        self.mock_wait_for.mock.assert_called_once_with(
+        config = CONF.openstack
+        self.mock_wait_for_status.mock.assert_called_once_with(
             environment,
             update_resource=self.mock_get_from_manager.mock.return_value,
             ready_statuses=["READY"],
